'''
Created on 10/01/2014

@author: MMPE
'''
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import numpy as np
from pandas import read_csv



def pandas_dat_ascii2bin(ascii_filename, bin_filename, ui):

    df = ui.exec_long_task("Reading ascii file", False, read_csv, ascii_filename, sep=" ", skipinitialspace=True, header=None)

    def compress(df, bin_filename):
        with open(bin_filename, 'wb') as outfile:
            scale_factors = []
            for _, sensor in df.iteritems():
                sf = sensor.abs().max() / 32000
                if sf > 0:
                    sensor /= sf
                np.round(sensor.values).astype(np.int16).tofile(outfile)
                scale_factors.append(sf)
        return np.array(scale_factors)
    #return compress(df, bin_filename)
    return ui.exec_long_task("Compress and save as binary", False, compress, df, bin_filename)




#!/usr/bin/python
#-----------------------------------------------------------------------------
# Author: David Verelst - david.verelst@gmail.com
# Version: 0.10 - 15/03/2010
# Version: 0.11 - 13/04/2010 : adding nodes down, offline to the total
#                                 overview of available nodes
# version: 0.20 - 17/11/2011 : major rewrite, support added for gorm
# version: 0.30 - 19/12/2012 : refactoring, added the node overview
# version: 0.40 - 26/03/2014 : removed Thyra, added support for Jess
#-----------------------------------------------------------------------------

import os
import socket

def print_dashboard(users, host, pbsnodes):

    # print nicely
    # the header
    #    ---------------------------------------------
    #         User  Running   Queued  Waiting    Other
    #    ---------------------------------------------
    #         jber        3        0        0        0

    print
    print ('-' * 54)
    print ('cpus'.rjust(18) + 'nodes'.rjust(9))
    print ('User'.rjust(9) + 'Running'.rjust(9) + 'Running'.rjust(9) \
        + 'Queued'.rjust(9) + 'Waiting'.rjust(9) + 'Other'.rjust(9))
    # nodeSum: overview (summation of all jobs) nodes per user:
    # nodeSum = [running, queued, waiting, other, cpus]
    nodeSum = [0, 0, 0, 0, 0]
    print ('-' * 54)
    # print all values in the table: the nodes used per user
    #userlist = users['users'].keys()
    #userlist.sort()
    for uid in sorted(users):

        # or get the unique nodes the user is on
        try:
            R = len(users[uid]['nodes'])
        except KeyError:
            # means it is not running yet but queued, waiting or otherwise
            R = users[uid]['R']
        Q = users[uid]['Q']
        W = users[uid]['W']
        O = users[uid]['E'] + users[uid]['H'] + users[uid]['T'] \
           + users[uid]['S'] + users[uid]['O'] + users[uid]['C']

        cpus = users[uid]['cpus']
        print (uid.rjust(9) + str(cpus).rjust(9) + str(R).rjust(9) \
            + str(Q).rjust(9) + str(W).rjust(9) + str(O).rjust(9))
        nodeSum[0] += R
        nodeSum[1] += Q
        nodeSum[2] += W
        nodeSum[3] += O
        nodeSum[4] += cpus

    nr_nodes = pbsnodes['nr_nodes']
    down = pbsnodes['down']
    others = pbsnodes['others']
    total_cpu = host['cpu_per_node'] * nr_nodes

    # the summed up for each node status (queued, running,...)
    print ('-' * 54)
    print ('total'.rjust(9) + str(nodeSum[4]).rjust(9) + str(nodeSum[0]).rjust(9) \
        + str(nodeSum[1]).rjust(9) + str(nodeSum[2]).rjust(9)\
        + str(nodeSum[3]).rjust(9))
    print ('-' * 54)
    print ('free'.rjust(9) + str(total_cpu - nodeSum[4]).rjust(9) \
        + str(nr_nodes - nodeSum[0] - others - down).rjust(9))
    print ('down'.rjust(9) + str(down).rjust(18))
    print ('-' * 54)
    print


def print_node_loading(users, host, nodes, nodesload):
    """
    Give an overview of how each node is loaded
    """

    if len(host) < 1:
        print ('It is very quit, nobody is working on the cluster.')
        return

    hostname = host['name']
    cpunode = host['cpu_per_node']

    print
    # print a header
    if hostname == 'gorm':
        print ('-' * 79)
        header = '|'.join([str(k).center(5) for k in range(1, 13, 1)]) + '|'
        print ('id'.center(5), header)
        print ('-' * 79)
    elif hostname == 'jess':
        print ('-' * 126)
        header = '|'.join([str(k).center(5) for k in range(1, 21, 1)]) + '|'
        print ('id'.center(5), header)
        print ('-' * 126)

    # print who is using the nodes
    for node in sorted(nodes):
        status = nodes[node]
        # now we have a list of user on this node
        try:
            users = sorted(nodesload[node])
            for kk in range(len(users), cpunode):
                users.append('')
            # limit uid names to 5 characters
            printlist = '|'.join([k[:5].center(5) for k in users]) + '|'
        # if it doesn't exist in the nodesload, just print the status
        except KeyError:
            printlist = status.center(5)

        print (node, printlist)

    # print a header
    if hostname == 'gorm':
        print ('-' * 79)
        print ('id'.center(5), header)
        print ('-' * 79)
    elif hostname == 'jess':
        print ('-' * 126)
        print ('id'.center(5), header)
        print ('-' * 126)
    #print


def parse_pbsnode_lall(output):
    """Parse output of pbsnodes -l all
    """
    # read the qstat output
    frees, exclusives, others, down = 0, 0, 0, 0
    nr_nodes = 0

    nodes = {}

    for k in output:
        if len(k) > 2:
            line = k.split()
            status = line[1]
            node = line[0].split('.')[0]

            if node.startswith('v-'):
                #host = 'valde'
                # uglye monkey patch: ignore any valde nodes
                continue

            #elif node.startswith('g-'):
                #host = 'gorm'
            #elif node.startswith('th-'):
                #host = 'thyra'

            if status == 'free':
                frees += 1
                nr_nodes += 1
            elif status == 'job-exclusive':
                exclusives += 1
                nr_nodes += 1
            elif status == 'down,offline':
                down += 1
            elif status == 'offline':
                down += 1
            elif status == 'down':
                down += 1
            else:
                others += 1

            nodes[node] = status

    #check = frees + exclusives + down + others

    pbsnodes = {'frees' : frees, 'nr_nodes' : nr_nodes, 'others' : others,
               'exclusives' : exclusives, 'down' : down}

    return pbsnodes, nodes


def parse_qstat_n1(output, hostname=None):
    """
    Parse the output of qstat -n1
    """

    # for global node usage, keep track of how many processes are running on
    # each of the nodes
    nodesload = {}
    # store it all in dictionaries
    host = {}
    users = {}
    # get the hostname
    if hostname is None:
        hostname = socket.gethostname()
    if 'jess' in hostname:
        host['name'] = 'jess'
        host['cpu_per_node'] = 20
    else:
        host['name'] = 'gorm'
        host['cpu_per_node'] = 12

    # take the available nodes in nr_nodes: it excludes the ones
    # who are down
    #queue['_total_cpu_'] = cpu_node*nr_nodes

    for line in output[5:]:
        if len(line.strip()) == 0:
            continue
        items = line.split()
        queue = items[2]

        # uglye monkey patch: ignore any valde nodes
        if queue == 'valdeq':
            continue

        jobid = items[0]
        # valid line starts with the jobid, which is an int
        jobid = jobid.split('.')[0]
        userid = items[1]
        # nr nodes used by the job
        job_nodes = int(items[5])
        # status of the job
        job_status = items[9]
        # is the user already in the queue dict?
        try:
            users[userid]['jobs'].append(jobid)
            users[userid][job_status] += job_nodes
        # if the user wasn't added before, create the sub dictionaries
        except KeyError:
            # initialise the users dictionary and job list
            users[userid] = dict()
            users[userid]['C'] = 0
            users[userid]['E'] = 0
            users[userid]['H'] = 0
            users[userid]['Q'] = 0
            users[userid]['R'] = 0
            users[userid]['T'] = 0
            users[userid]['W'] = 0
            users[userid]['S'] = 0
            users[userid]['O'] = 0
            users[userid]['cpus'] = 0
            users[userid]['jobs'] = []
            users[userid]['nodes'] = set()
            # set the values
            users[userid]['jobs'].append(jobid)
            users[userid][job_status] += job_nodes

        if job_status == 'R':
            # each occurance of the node name is seprated by a + and
            # indicates a process running on a CPU of that node
            nodes = items[11].split('+')
            # TODO: take into account cpu number for jess: j-304/5
            # on jess, the cpu number of the node is indicated, ignore for now
            # FIXME: host name can be jess or j-
            if host['name'].startswith('jess'):
                for i, node in enumerate(nodes):
                    nodes[i] = node.split('/')[0]
            # keep track of the number of processes the user running
            users[userid]['cpus'] += len(nodes)
            # for the number of used nodes, keep track of the unique nodes used
            users[userid]['nodes'].update(set(nodes))
            # remember all the nodes the user is on in a dictionary
            for node in nodes:
                try:
                    nodesload[node].append(userid)
                except KeyError:
                    nodesload[node] = [userid]

    return users, host, nodesload

# FIXME: counts diffferent compared to launch.py....
def count_cpus(users, host, pbsnodes):
    """
    See how many cpu's are actually free
    """
    nodeSum = {'R':0, 'Q':0, 'W':0, 'O':0, 'used_cpu':0, 'H':0}

    for uid in users:

        # or get the unique nodes the user is on
        try:
            nodeSum['R'] = len(users[uid]['nodes'])
        except KeyError:
            # means it is not running yet but queued, waiting or otherwise
            nodeSum['R'] = users[uid]['R']

        nodeSum['Q'] += users[uid]['Q']
        nodeSum['W'] += users[uid]['W']
        nodeSum['H'] += users[uid]['H']
        nodeSum['used_cpu'] += users[uid]['cpus']
        nodeSum['O'] += users[uid]['E'] + users[uid]['T'] + users[uid]['S'] \
                      + users[uid]['O'] + users[uid]['C']

    # free cpus
    down_cpu = host['cpu_per_node'] * pbsnodes['down']
    total_cpu = host['cpu_per_node'] * pbsnodes['nr_nodes']
    cpu_free = total_cpu - down_cpu - nodeSum['used_cpu']

    return cpu_free, nodeSum


PBS_TEMP = """
### Standard Output
#PBS -N [jobname]
#PBS -o ./[pbs_out_file]
### Standard Error
#PBS -e ./[pbs_err_file]
#PBS -W umask=003
### Maximum wallclock time format HOURS:MINUTES:SECONDS
#PBS -l walltime=[walltime]
#PBS -lnodes=[lnodes]:ppn=[ppn]
### Queue name
#PBS -q [queue]
### Browse to current working dir
cd $PBS_O_WORKDIR
pwd
### ===========================================================================
### set environment variables, activate python virtual environment, execute
[commands]

### ===========================================================================
### wait for jobs to finish
wait
exit
"""
# TODO: this is very similar compared to what happens in qsub-wrap
def create_input(walltime='00:59:59', queue='xpresq', pbs_in='pbs_in/', ppn=1,
                 pbs_out='pbs_out/', jobname=None, commands=None, lnodes=1):
    """
    Create a PBS script for a command. Optionally, define a python environment.
    """

    pbs_err_file = os.path.join(pbs_out, jobname + '.err')
    pbs_out_file = os.path.join(pbs_out, jobname + '.out')
    pbs_in_file = os.path.join(pbs_in, jobname + '.pbswrap')

    pbs_script = PBS_TEMP
    pbs_script = pbs_script.replace('[jobname]', jobname)
    pbs_script = pbs_script.replace('[pbs_out_file]', pbs_out_file)
    pbs_script = pbs_script.replace('[pbs_err_file]', pbs_err_file)
    pbs_script = pbs_script.replace('[walltime]', walltime)
    pbs_script = pbs_script.replace('[lnodes]', str(lnodes))
    pbs_script = pbs_script.replace('[ppn]', str(ppn))
    pbs_script = pbs_script.replace('[queue]', queue)
    pbs_script = pbs_script.replace('[commands]', commands)

    print ('following commands will be executed on the cluster:')
    print ('%s' % (commands))

    # make sure a pbs_in and pbs_out directory exists
    if not os.path.exists(pbs_in):
        os.makedirs(pbs_in)
    if not os.path.exists(pbs_out):
        os.makedirs(pbs_out)

    # write the pbs_script
    FILE = open(pbs_in_file, 'w')
    FILE.write(pbs_script)
    FILE.close()

    return pbs_in_file


if __name__ == '__main__':

    pass



import glob
import unittest
import os

def module_strings():
    test_file_paths = glob.glob('test_*.py')
    #test_file_strings.extend(glob.glob('../wetb/**/test_*.py'))
    #for root,_,_ in os.walk("../wetb/"):

    for folder, _, _ in os.walk('../wetb'):
        test_file_paths.extend(glob.glob(os.path.join(folder, "tests/test_*.py")))
    return [s[3:len(s) - 3].replace(os.path.sep, ".") for s in test_file_paths]


def suite():
    try:
        suites = []
        for s in module_strings():
            suites.append(unittest.defaultTestLoader.loadTestsFromName(s))
    except:
        print ("Failed to import '%s'" % s)

    return unittest.TestSuite(suites)

#no_tests = 0
#all = set()
#all_modules = set()
#for s in module_strings():
#    if s.split('.')[-1] in all_modules:
#        print ("!!!!!%s already loaded" % s)
#    all_modules.add(s.split('.')[-1])
#    m = __import__(s, {}, {}, "*")
#    cls = [t for t in m.__dict__.keys() if t.lower().startswith('test') and t != 'TestCaseAppFunc' and t != "TestCase" and t.strip()[0] != "#"]
#
#    test_funcs = [t for t in dir(m.__dict__[cls[0]]) if t.lower().startswith('test') and t != 'testfilepath']
#    for t in test_funcs:
#        if t in all:
#            print ("!!!!!!!!! %s already present" % t)
#            pass
#        else:
#            all.add(t)
#    no_tests += len(test_funcs)
#    print ("%-40s" % s, len(test_funcs), "\t\t", test_funcs)
#print ("Number of tests: ", no_tests, len(all), len(module_strings()))




if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    test_suite = suite()
    runner.run(test_suite)
else:
    # for run as pydev unit-test
    try:
        for mstr in module_strings():
            __import__(mstr, {}, {}, "*")
            exec("from %s import *" % mstr)
    except Exception as e:
        for mstr in module_strings():
            print (mstr)
            __import__(mstr, {}, {}, "*")
            exec("from %s import *" % mstr)



from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from builtins import map
from future import standard_library
standard_library.install_aliases()
import numpy as np

def rad(deg):
    return deg * np.pi / 180

def deg(rad):
    return rad / np.pi * 180

def sind(dir_deg):
    return np.sin(rad(dir_deg))

def cosd(dir_deg):
    return np.cos(rad(dir_deg))

def tand(dir_deg):
    return np.tan(rad(dir_deg))

def mean_deg(dir, axis=0):
    """Mean of angles in degrees

    Parameters
    ----------
    dir : array_like
        Angles in degrees
    axis : int
        if dir is 2d array_like, axis defines which axis to take the mean of

    Returns
    -------
    mean_deg : float
        Mean angle
    """
    return deg(mean_rad(rad(dir), axis))

def mean_rad(dir, axis=0):
    """Mean of angles in radians

    Parameters
    ----------
    dir : array_like
        Angles in radians
    axis : int
        if dir is 2d array_like, axis defines which axis to take the mean of

    Returns
    -------
    mean_rad : float
        Mean angle
    """
    return np.arctan2(np.nanmean(np.sin(dir[:]), axis), np.nanmean(np.cos(dir[:]), axis))


def std_deg(dir):
    """Standard deviation of angles in degrees

    Parameters
    ----------
    dir : array_like
        Angles in degrees

    Returns
    -------
    std_deg : float
        standard deviation
    """
    return deg(std_rad(rad(dir)))

def std_rad(dir):
    """Standard deviation of angles in radians

    Parameters
    ----------
    dir : array_like
        Angles in radians

    Returns
    -------
    std_rad : float
        standard deviation
    """
    return np.sqrt(1 - (np.nanmean(np.sin(dir)) ** 2 + np.nanmean(np.cos(dir)) ** 2))

def rpm2rads(rpm):
    return rpm * 2 * np.pi / 60


def rads2rpm(rads):
    return rads/ (2 * np.pi) * 60


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Setup file for wetb.
"""

import os
import sys
from setuptools import setup, find_packages
import wetb
__version__ = wetb.__version__

try:
    from pypandoc import convert_file
    read_md = lambda f: convert_file(f, 'rst', format='md')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

import numpy as np
from distutils.extension import Extension
from Cython.Distutils import build_ext


def setup_package():

    ex_info = [('wetb.fatigue_tools.rainflowcounting', ['pair_range', 'peak_trough', 'rainflowcount_astm']),
			   ('wetb.signal.filters', ['cy_filters'])]
    extlist = [Extension('%s.%s' % (module, n),
                         [os.path.join(module.replace(".","/"), n)+'.pyx'],
                         include_dirs=[np.get_include()]) for module, names in ex_info for n in names]

    needs_sphinx = {'build_sphinx', 'upload_docs'}.intersection(sys.argv)
    sphinx = ['sphinx'] if needs_sphinx else []
    setup(setup_requires=['six'] + sphinx,
          cmdclass = {'build_ext': build_ext},
          ext_modules = extlist,
          long_description=read_md('README.md'),
          version=__version__,
          packages=find_packages(),
          )


if __name__ == "__main__":
    setup_package()



'''
Created on 05/11/2015

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import unittest
import os
import filecmp
import shutil
from zipfile import ZipFile

import numpy as np
import pandas as pd

from wetb.prepost import dlctemplate as tmpl
from wetb.prepost import Simulations as sim
from wetb.prepost.misc import DictDiff


class Template(unittest.TestCase):
    def setUp(self):
        self.basepath = os.path.dirname(__file__)


class TestErrorLogs(Template):

    def test_loganalysis(self):
        # select a few log cases and do the analysis, save to csv and convert
        # saved result to DataFrame
        pass


class TestGenerateInputs(Template):

    def test_launch_dlcs_excel(self):
        # manually configure paths, HAWC2 model root path is then constructed as
        # p_root_remote/PROJECT/sim_id, and p_root_local/PROJECT/sim_id
        # adopt accordingly when you have configured your directories differently
        p_root = os.path.join(self.basepath, 'data/')
        # project name, sim_id, master file name
        tmpl.PROJECT = 'demo_dlc'
        tmpl.MASTERFILE = 'demo_dlc_master_A0001.htc'
        # MODEL SOURCES, exchanche file sources
        tmpl.P_RUN = os.path.join(p_root, tmpl.PROJECT, 'remote/')
        tmpl.P_SOURCE = os.path.join(p_root, tmpl.PROJECT, 'source/')
        # location of the master file
        tmpl.P_MASTERFILE = os.path.join(p_root, tmpl.PROJECT,
                                         'source', 'htc', '_master/')
        # location of the pre and post processing data
        tmpl.POST_DIR = os.path.join(p_root, tmpl.PROJECT, 'remote',
                                     'prepost/')

        # make sure the remote dir is empty so a test does not pass on data
        # generated during a previous cycle
        if os.path.exists(os.path.join(p_root, tmpl.PROJECT, 'remote')):
            shutil.rmtree(os.path.join(p_root, tmpl.PROJECT, 'remote'))

        tmpl.force_dir = tmpl.P_RUN
        tmpl.launch_dlcs_excel('remote', silent=True, runmethod='gorm',
                               pbs_turb=True, zipchunks=True,
                               postpro_node_zipchunks=False,
                               postpro_node=False)

        def cmp_dir(dir1, dir2):
            lst1, lst2 = map(os.listdir, (dir1, dir2))
            self.assertEqual(";".join(lst1), ";".join(lst2))
            for f1, f2 in zip(lst1, lst2):
                if f1.endswith(".zip") or f1.endswith(".xlsx"):
                    continue
                if os.path.isdir(os.path.join(dir1, f1)):
                    cmp_dir(os.path.join(dir1, f1), os.path.join(dir2, f2))
                else:
                    try:
                        with open(os.path.join(dir1, f1)) as fid1:
                            l1 = fid1.readlines()
                        with open(os.path.join(dir2, f2)) as fid2:
                            l2 = fid2.readlines()

                        self.assertEqual(len(l1), len(l2))
                        self.assertTrue(all([l1_ == l2_ for l1_, l2_ in zip(l1, l2)]))
                    except:
                        print("=" * 30)
                        print(os.path.join(dir1, f1))
                        print(os.path.join(dir2, f2))
                        print(dir1[[d1 != d2 for d1, d2 in zip(dir1, dir2)].index(True):])
                        print(f1)
                        for i in range(len(l1)):
                            if l1[i] != l2[i]:
                                print("%03d, rem: %s" % (i, l1[i].strip()))
                                print("%03d, ref: %s" % (i, l2[i].strip()))
                                print()
                        raise

        # we can not git check-in empty dirs so we can not compare the complete
        # directory structure withouth manually creating the empty dirs here
        for subdir in ['control', 'data', 'htc', 'pbs_in', 'pbs_in_turb',
                       'htc/_master', 'htc/dlc01_demos', 'pbs_in/dlc01_demos',
                       'zip-chunks-jess', 'zip-chunks-gorm']:
            remote = os.path.join(p_root, tmpl.PROJECT, 'remote', subdir)
            ref = os.path.join(p_root, tmpl.PROJECT, 'ref', subdir)
            # the zipfiles are taken care of separately
            ignore = ['remote_chnk_00000.zip']
            cmp = filecmp.dircmp(remote, ref, ignore=ignore)
            cmp_dir(remote, ref)
            self.assertEqual(len(cmp.diff_files), 0,
                             "{} {}".format(subdir, cmp.diff_files))
            self.assertEqual(len(cmp.right_only), 0,
                             "{} {}".format(subdir, cmp.right_only))
            self.assertEqual(len(cmp.left_only), 0,
                             "{} {}".format(subdir, cmp.left_only))

        # compare the zip files
        for fname in ['demo_dlc_remote.zip',
                      'zip-chunks-gorm/remote_chnk_00000.zip',
                      'zip-chunks-jess/remote_chnk_00000.zip']:
            remote = os.path.join(p_root, tmpl.PROJECT, 'remote', fname)
            ref = os.path.join(p_root, tmpl.PROJECT, 'ref', fname)

            with ZipFile(remote) as zrem, ZipFile(ref) as zref:
                self.assertEqual(len(zrem.infolist()), len(zref.infolist()))
                frem = {f.filename:f.file_size for f in zrem.infolist()}
                fref = {f.filename:f.file_size for f in zref.infolist()}
                dd = DictDiff(frem, fref)
                self.assertEqual(len(dd.added()), 0,
                                 "{} {}".format(fname, dd.added()))
                self.assertEqual(len(dd.removed()), 0,
                                 "{} {}".format(fname, dd.removed()))
                self.assertEqual(len(dd.changed()), 0,
                                 "{} {}".format(fname, dd.changed()))

        # for the pickled file we can just read it
        remote = os.path.join(p_root, tmpl.PROJECT, 'remote', 'prepost')
        ref = os.path.join(p_root, tmpl.PROJECT, 'ref', 'prepost')
        cmp = filecmp.cmp(os.path.join(remote, 'remote_tags.txt'),
                          os.path.join(ref, 'remote_tags.txt'), shallow=False)
        self.assertTrue(cmp)
#        with open(os.path.join(remote, 'remote.pkl'), 'rb') as FILE:
#            pkl_remote = pickle.load(FILE)
#        with open(os.path.join(ref, 'remote.pkl'), 'rb') as FILE:
#            pkl_ref = pickle.load(FILE)
#        self.assertTrue(pkl_remote == pkl_ref)


class TestFatigueLifetime(Template):

    def test_leq_life(self):
        """Verify if prepost.Simulation.Cases.fatigue_lifetime() returns
        the expected life time equivalent load.
        """
        # ---------------------------------------------------------------------
        # very simple case
        cases = {'case1':{'[post_dir]':'no-path', '[sim_id]':'A0'},
                 'case2':{'[post_dir]':'no-path', '[sim_id]':'A0'}}
        cc = sim.Cases(cases)

        fh_list = [('case1', 10/3600), ('case2', 20/3600)]
        dfs = pd.DataFrame({'m=1.0' : [2, 3],
                            'channel' : ['channel1', 'channel1'],
                            '[case_id]' : ['case1', 'case2']})
        neq_life = 1.0
        df_Leq = cc.fatigue_lifetime(dfs, neq_life, fh_lst=fh_list,
                                     save=False, update=False, csv=False,
                                     xlsx=False, silent=False)
        np.testing.assert_allclose(df_Leq['m=1.0'].values, 2*10 + 3*20)
        self.assertTrue(df_Leq['channel'].values[0]=='channel1')

        # ---------------------------------------------------------------------
        # slightly more complicated
        neq_life = 3.0
        df_Leq = cc.fatigue_lifetime(dfs, neq_life, fh_lst=fh_list,
                                     save=False, update=False, csv=False,
                                     xlsx=False, silent=False)
        np.testing.assert_allclose(df_Leq['m=1.0'].values,
                                   (2*10 + 3*20)/neq_life)

        # ---------------------------------------------------------------------
        # a bit more complex and also test the sorting of fh_lst and dfs
        cases = {'case1':{'[post_dir]':'no-path', '[sim_id]':'A0'},
                 'case2':{'[post_dir]':'no-path', '[sim_id]':'A0'},
                 'case3':{'[post_dir]':'no-path', '[sim_id]':'A0'},
                 'case4':{'[post_dir]':'no-path', '[sim_id]':'A0'}}
        cc = sim.Cases(cases)

        fh_list = [('case3', 10/3600), ('case2', 20/3600),
                   ('case1', 50/3600), ('case4', 40/3600)]
        dfs = pd.DataFrame({'m=3.0' : [2, 3, 4, 5],
                            'channel' : ['channel1']*4,
                            '[case_id]' : ['case4', 'case2', 'case3', 'case1']})
        neq_life = 5.0
        df_Leq = cc.fatigue_lifetime(dfs, neq_life, fh_lst=fh_list,
                                     save=False, update=False, csv=False,
                                     xlsx=False, silent=False)
        expected = ((2*2*2*40 + 3*3*3*20 + 4*4*4*10 + 5*5*5*50)/5)**(1/3)
        np.testing.assert_allclose(df_Leq['m=3.0'].values, expected)

        # ---------------------------------------------------------------------
        # more cases and with sorting
        base = {'[post_dir]':'no-path', '[sim_id]':'A0'}
        cases = {'case%i' % k : base for k in range(50)}
        cc = sim.Cases(cases)
        # reverse the order of how they appear in dfs and fh_lst
        fh_list = [('case%i' % k, k*10/3600) for k in range(49,-1,-1)]
        dfs = pd.DataFrame({'m=5.2' : np.arange(1,51,1),
                            'channel' : ['channel1']*50,
                            '[case_id]' : ['case%i' % k for k in range(50)]})
        df_Leq = cc.fatigue_lifetime(dfs, neq_life, fh_lst=fh_list,
                                     save=False, update=False, csv=False,
                                     xlsx=False, silent=False)
        expected = np.sum(np.power(np.arange(1,51,1), 5.2)*np.arange(0,50,1)*10)
        expected = np.power(expected/neq_life, 1/5.2)
        np.testing.assert_allclose(df_Leq['m=5.2'].values, expected)


if __name__ == "__main__":
    unittest.main()



'''
Created on 16/06/2014

@author: MMPE
'''
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

from scipy.optimize.optimize import fmin
import numpy as np


def _z_u(z_u_lst):
    z = np.array([z for z, _ in z_u_lst])
    u = np.array([np.mean(np.array([u])[:]) for _, u in z_u_lst])
    return z, u

def power_shear(alpha, z_ref, u_ref):
    """Power shear

    Parameters
    ----------
    alpha : int or float
        The alpha shear parameter
    z_ref : int or float
        The reference height
    u_ref : int or float
        The wind speed of the reference height

    Returns
    -------
    power_shear : function
        Function returning for wsp at input heights: f(height) -> wsp

    Example
    --------
    >>> power_shear(.5, 70, 9)([20,50,70])
    [ 4.81070235  7.60638829  9.        ]
    """
    return lambda z : u_ref * (np.array(z) / z_ref) ** alpha


def fit_power_shear(z_u_lst):
    """Estimate power shear parameter, alpha, from the mean wind at hub height and one additional height

    Parameters
    ----------
    z_u_lst : [(z_ref, u_z_ref), (z1, u_z1)]
        - z_ref: Reference height\n
        - u_z_ref: Wind speeds or mean wind speed at z_ref
        - z1: another height
        - u_z1: Wind speeds or mean wind speeds at z1

    Returns
    -------
    alpha : float
        power shear parameter

    Example
    --------
    >>> fit_power_shear([(85, 9.0), (21, 4.47118)])
    0.50036320835
    """
    z, u = _z_u(z_u_lst)
    z_hub, u_hub = z[0], u[0]
    alpha, _ = np.polyfit(np.log(z / z_hub), np.log(u / u_hub), 1)
    return alpha

def fit_power_shear_ref(z_u_lst, z_ref, plt=None):
    """Estimate power shear parameter, alpha, from two or more specific reference heights using polynomial fit.

    Parameters
    ----------
    z_u_lst : [(z1, u_z1), (z2, u_z2),...]
        - z1: Some height
        - u_z1: Wind speeds or mean wind speed at z1
        - z2: another height
        - u_z2: Wind speeds or mean wind speeds at z2
    z_ref : float or int
        Reference height (hub height)
    plt : matplotlib.pyplot (or similar) or None
        Used to plot result if not None

    Returns
    -------
    alpha : float
        power shear parameter
    u_ref : float
        Wind speed at reference height

    Example
    --------
    >>> fit_power_shear_ref([(85, 8.88131), (21, 4.41832)],  87.13333)
    [ 0.49938238  8.99192568]
    """
    def shear_error(x, z_u_lst, z_ref):
        alpha, u_ref = x
        return np.nansum([(u - u_ref * (z / z_ref) ** alpha) ** 2 for z, u in z_u_lst])
    z_u_lst = [(z, np.mean(u)) for z, u in z_u_lst]
    alpha, u_ref = fmin(shear_error, (.1, 10), (z_u_lst, z_ref), disp=False)
    if plt:
        z, u = list(zip(*z_u_lst))
        plt.plot(u, z, '.')
        z = np.linspace(min(z), max(z), 100)
        plt.plot(power_shear(alpha, z_ref, u_ref)(z), z)
        plt.margins(.1)
    if alpha==.1 and u_ref==10: # Initial conditions
        return np.nan, np.nan
    return alpha, u_ref



def log_shear(u_star, z0):
    """logarithmic shear

    Parameters
    ----------
    u_star : int or float
        Friction velocity
    z0 : int or float
        Surface roughness [m]

    Returns
    -------
    log_shear : function f(z,L=None) -> wsp
        z : int, float or array_like
            The heights of interest
        L : int, float or array_like, optional
            The corresponding Monin-Obukhov length

    Example
    --------
    >>> shear = log_shear(1, 10)
    >>> shear([20, 50, 70])
    [ 1.73286795  4.02359478  4.86477537]
    """
    K = 0.4  # von Karmans constant
    def log_shear(z,Obukhov_length=None):
        if Obukhov_length is None:
            return u_star / K * (np.log(np.array(z) / z0))
        else:
            return u_star / K * (np.log(z / z0) - stability_term(z, z0, Obukhov_length))
    return log_shear

def stability_term(z, z0, L):
    """Calculate the stability term for the log shear

    Not validated!!!
    """
    zL = z / L
    phi_us = lambda zL : (1 + 16 * np.abs(zL)) ** (-1 / 4)  #unstable
    phi_s = lambda zL : 1 + 5 * zL  # stable
    phi = np.zeros_like(zL) + np.nan

    for m, f in [(((-2 <= zL) & (zL <= 0)), phi_us), (((0 < zL) & (zL <= 1)), phi_s)]:
        phi[m] = f(zL[m])
    f = L / z * (1 - phi)
    psi = np.cumsum(np.r_[0, (f[1:] + f[:-1]) / 2 * np.diff(z / L)])
    return psi


def fit_log_shear(z_u_lst, include_R=False):
    """Estimate log shear parameter, u_star and z0

    Parameters
    ----------
    z_u_lst : [(z1, u_z1), (z2, u_z2),...]
        - z1: Some height
        - u_z1: Wind speeds or mean wind speed at z1
        - z2: another height
        - u_z2: Wind speeds or mean wind speeds at z2
    include_R : boolean, optional
        If True, the R^2 error is returned

    Returns
    -------
    u_star : float
        friction velocity
    z0 : float
        Surface roughness [m]

    Example
    --------
    >>> fit_log_shear([(85, 8.88131), (21, 4.41832)])
    [ 0.49938238  8.99192568]
    """
#    def shear_error(x, z_u_lst):
#        u_star, z0 = x
#        return np.sum([(np.mean(u) - log_shear(u_star, z0, z)) ** 2 for z, u in z_u_lst])
#    return fmin(shear_error, (1, 1), (z_u_lst,), disp=False)
    z, U = _z_u(z_u_lst)
    a, b = np.polyfit(np.log(z), U, 1)
    kappa = 0.4
    if include_R:
        return a * kappa, np.exp(-b / a), sum((U - (a * np.log(z) + b)) ** 2)
    return a * kappa, np.exp(-b / a)

if __name__ == '__main__':
    from matplotlib.pyplot import plot, show
    z = np.arange(0, 211)
    for alpha, c in zip([0.00001, 1, 2], ['r', 'b', 'g']):
        u = power_shear(alpha, 120, 10)(z)
        plot(u, z, c)
        plot(u.mean(), 120, c + '.')

    plot([8.5, 11], [120, 120])
    show()



'''
Created on 28. jul. 2017

@author: mmpe
'''
import os
import subprocess


def _run_git_cmd(cmd, git_repo_path=None):
    git_repo_path = git_repo_path or os.getcwd()
    if not os.path.isdir(os.path.join(git_repo_path, ".git")):
        raise Warning("'%s' does not appear to be a Git repository." % git_repo_path)
    try:
        process = subprocess.Popen(cmd,
                                   stdout=subprocess.PIPE,
                                   universal_newlines=True,
                                   cwd=os.path.abspath(git_repo_path))
        stdout = process.communicate()[0]
        if process.returncode != 0:
            raise EnvironmentError()
        return stdout.strip()

    except EnvironmentError as e:
        raise e
        raise Warning("unable to run git")


def get_git_version(git_repo_path=None):
    cmd = ["git", "describe", "--tags", "--dirty", "--always"]
    return _run_git_cmd(cmd, git_repo_path)


def get_tag(git_repo_path=None):
    return _run_git_cmd(['git', 'describe', '--tags', '--abbrev=0'], git_repo_path)


def set_tag(tag, push, git_repo_path=None):
    _run_git_cmd(["git", "tag", tag], git_repo_path)
    if push:
        _run_git_cmd(["git", "push"], git_repo_path)
        _run_git_cmd(["git", "push", "--tags"], git_repo_path)


def update_git_version(version_module, git_repo_path=None):
    """Update <version_module>.__version__ to git version"""

    version_str = get_git_version(git_repo_path)
    assert os.path.isfile(version_module.__file__)
    with open(version_module.__file__, "w") as fid:
        fid.write("__version__ = '%s'" % version_str)

    # ensure file is written, closed and ready
    with open(version_module.__file__) as fid:
        fid.read()
    return version_str

def write_vers(vers_file='wetb/__init__.py'):
    version = get_tag(os.getcwd())
    print('Writing version: {} in {}'.format(version, vers_file))
    with open(vers_file, 'r') as f:
        lines = f.readlines()
    for n,l in enumerate(lines):
        if l.startswith('__version__'):
            lines[n] = "__version__ = '{}'\n".format(version[1:])
    with open(vers_file, 'w') as f:
        f.write(''.join(lines))
		
def rename_dist_file():
    for f in os.listdir('dist'):
        if f.endswith('whl'):
            split = f.split('linux')
            new_name = 'manylinux1'.join(split)
            old_path = os.path.join('dist',f)
            new_path = os.path.join('dist',new_name)
            os.rename(old_path, new_path)
    

def main():
    """Example of how to run (pytest-friendly)"""
    if __name__ == '__main__':
#        pass
#        import version
#        import app_utils
#        git_path = os.path.dirname(app_utils.__file__) + "/../"
#        update_git_version(version, git_path)
#        tag = get_tag(os.getcwd())
#        print(tag)
#        version = get_git_version(os.getcwd())
#        print(version)
#        rename_dist_file()
        write_vers()


main()



'''
Created on 21/07/2014

@author: mmpe
'''
import numpy as np


def transformation_matrix(angles, xyz):
    """Create Transformation matrix(es)
    !!!Note that the columns of the returned matrix(es) is original(unrotate) xyz-axes in rotated coordinates\n
    !!!Multiplying a this matrix by a vector rotates the vector -angle radians (in right handed terminology).

    Parameters
    ----------
    angles : int, float or array_like
        Angle(s) for rotation matrix(es). \n
        One rotation matrix will be returned for each angle
    xyz : {0,1,2}
        - 0: rotation around x(first) axis\n
        - 1: rotation around y(second) axis\n
        - 2: rotation around z(third) axis\n

    Returns
    -------
    transformation_matrix : array_like, shape = (no_angles, 3,3)
        Rotation matrix(es)
    """
    indexes = [(0, 4, 8, 5, 7), (4, 0, 8, 6, 2), (8, 0, 4, 1, 3)]  # 1, cos,cos,sin,-sin for x,y and z rotation matrix
    if isinstance(angles, (int, float)):
        n = 1
    else:
        n = len(angles)
    m = np.zeros(n * 9, np.float)
    cosx = np.cos(angles)
    sinx = np.sin(angles)
    m[indexes[xyz][0]::9] = 1
    m[indexes[xyz][1]::9] = cosx
    m[indexes[xyz][2]::9] = cosx
    m[indexes[xyz][3]::9] = sinx
    m[indexes[xyz][4]::9] = -sinx
    return m.reshape(n, 3, 3)

def rotmat(angles, xyz):
    """Create rotation matrix(es)
    !!!Note that the columns of the returned matrix(es) is rotated xyz-axes in original(unrotated) coordinates\n
    Multiplying a this matrix by a vector rotates the vector +angle radians (in right handed terminology).
    Parameters
    ----------
    angles : int, float or array_like
        Angle(s) for rotation matrix(es). \n
        Note that
        One rotation matrix will be returned for each angle
    xyz : {0,1,2}
        - 0: rotation around x(first) axis\n
        - 1: rotation around y(second) axis\n
        - 2: rotation around z(third) axis\n

    Returns
    -------
    transformation_matrix : array_like, shape = (no_angles, 3,3)
        Rotation matrix(es)
    """
    indexes = [(0, 4, 8, 5, 7), (4, 0, 8, 6, 2), (8, 0, 4, 1, 3)]  # 1, cos,cos,sin,-sin for x,y and z rotation matrix
    if isinstance(angles, (int, float)):
        n = 1
    else:
        n = len(angles)
    m = np.zeros(n * 9, np.float)
    cosx = np.cos(angles)
    sinx = np.sin(angles)
    m[indexes[xyz][0]::9] = 1
    m[indexes[xyz][1]::9] = cosx
    m[indexes[xyz][2]::9] = cosx
    m[indexes[xyz][3]::9] = -sinx
    m[indexes[xyz][4]::9] = sinx
    return m.reshape(n, 3, 3)

def mdot(m1, m2):
    """Multiplication of matrix pairs

    Parameters
    ----------
    m1 : array_like shape = (i, n, m)
        First matrix set, i.e. i matrixes, size n x m\n
        i must be equal to j or 1
    m2 : array_like, shape = (j,m,p)
        Second matrix set, i.e. j matrixes, size m x p\n
        j must be equal to i or 1

    Returns
    -------
    mdot : array_like, shape(max(i,j),n,p)
        Matrix products
    """
    if m1.shape[0] == 1 and m2.shape[0] == 1:
        return np.array([np.dot(m1[0], m2[0])])
    elif m1.shape[0] > 1 and m2.shape[0] == 1:
        mprod = np.empty_like(m1)
        for i in range(m1.shape[0]):
            mprod[i, :] = np.dot(m1[i, :], m2[0])
    elif m1.shape[0] == 1 and m2.shape[0] > 1:
        mprod = np.empty_like(m2)
        for i in range(m2.shape[0]):
            mprod[i, :] = np.dot(m1[0], m2[i, :])
    elif m1.shape[0] > 1 and m2.shape[0] > 1 and m1.shape[0] == m2.shape[0]:
        mprod = np.empty_like(m1)
        for i in range(m1.shape[0]):
            mprod[i, :] = np.dot(m1[i, :], m2[i, :])
    else:
        raise Exception("m1 and m2 must have same first dimension or one of the matrices must have first dimension 1")
    return mprod


def dots(rotmats, v):
    """Rotate vector(s) v by rotation matrix(es)

    Parameters
    ----------
    rotmats : array_like, shape=(i,3,3) or list of array_like, shape=(i,3,3)
        if list of array_like, rotation matrixes are recursively reduced by multiplication\n
        i must be 1 or equal to j
    vector : array_like, shape=(j,3)
        vectors to rotate

    Returns
    -------
    dots : array_like, shape=(j,3)
    """
    if isinstance(rotmats, list):
        if len(rotmats) > 1:
            rotmats_red = [mdot(rotmats[0], rotmats[1])]
            rotmats_red.extend(rotmats[2:])
            return dots(rotmats_red, v)
        else:
            m = rotmats[0]
    else:
        m = rotmats
    if len(v.shape) == 1:
        v = np.array([v]).T

    if m.shape[0] == 1:
        return np.dot(m[0], v)
    else:
        if m.shape[0] != v.shape[1]:
            raise Exception("m must have same dimension as v has number of columns")
        res = np.zeros_like(v, dtype=np.float)
        for i in range(v.shape[1]):
            res[:, i] = np.dot(m[i], v[:, i])
        return res


def rotate(rotmats, v):
    global x, y, z
    v = np.array(v)
    if isinstance(rotmats, (list, tuple)):
        if len(rotmats) > 1:
            rotmats_red = [mdot(rotmats[0], rotmats[1])]
            rotmats_red.extend(rotmats[2:])
            return rotate(rotmats_red, v)
        else:
            rotmats = rotmats[0]
    assert rotmats.shape[-1] == rotmats.shape[-2] == v.shape[-1] == 3, "rotmats is %d x %x and v is i x %d, but must be 3" % (rotmats.shape[-1], rotmats.shape[-2], v.shape[-1])

#    if isinstance(v, tuple):
#        v = np.array([v])
    n = (1, v.shape[0])[len(v.shape) == 2]

    if rotmats.shape[0] == 1:
        return np.dot(rotmats[0], v.T).T
    else:
        if v.shape[0] != rotmats.shape[0]:
            raise Exception("V and rotmats must have same first dimension")
        v = v.T
        v_rot = np.zeros_like(v, dtype=np.float)
        for i in range(n):
            v_rot[:, i] = np.dot(rotmats[i], v[:, i])
        return v_rot.T

def rotate_x(v, angle):
    """Rotate vector(s) around x axis

    Parameters
    ---------
    v : array_like, shape=(3,) or shape=(1-N,3)
        Vector(s) to rotate
    angle : int, float or array_like shape=(1,) or shape=(N,)
        Angle(s) [rad] to rotate

    Returns
    -------
    y : array_like
        Rotated vector(s)
    """
    return _rotate(v, angle, lambda x, y, z, cos, sin : [x, cos * y - sin * z, sin * y + cos * z])

def rotate_y(v, angle):
    """Rotate vector(s) around y axis

    Parameters
    ---------
    v : array_like, shape=(3,) or shape=(1-N,3)
        Vector(s) to rotate
    angle : int, float or array_like shape=(1,) or shape=(N,)
        Angle(s) [rad] to rotate

    Returns
    -------
    y : array_like
        Rotated vector(s)
    """
    return _rotate(v, angle, lambda x, y, z, cos, sin : [cos * x + sin * z, y, -sin * x + cos * z])

def rotate_z(v, angle):
    """Rotate vector(s) around z axis

    Parameters
    ---------
    v : array_like, shape=(3,) or shape=(1-N,3)
        Vector(s) to rotate
    angle : int, float or array_like shape=(1,) or shape=(N,)
        Angle(s) [rad] to rotate

    Returns
    -------
    y : array_like
        Rotated vector(s)
    """
    return _rotate(v, angle, lambda x, y, z, cos, sin : [cos * x - sin * y, sin * x + cos * y, z])

def _rotate(v, angle, rotfunc):
    angle = np.atleast_1d(angle)
    cos, sin = np.cos(angle), np.sin(angle)
    v = np.array(v)
    if len(v.shape) == 1:
        assert angle.shape[0] == 1
        assert v.shape[0] == 3
        return np.array(rotfunc(v[0], v[1], v[2], cos, sin), dtype=np.float).T
    else:
        assert angle.shape[0] == 1 or angle.shape[0] == v.shape[0]
        assert v.shape[1] == 3
        return np.array(rotfunc(v[:, 0], v[:, 1], v[:, 2], cos, sin), dtype=np.float).T



import pandas as pd
from wetb.hawc2.htc_file import HTCFile
from wetb.hawc2.tests import test_files
import os
import itertools
import importlib
from pandas.core.base import PandasObject
import numpy as np


class HAWC2InputWriter():

    def __init__(self, base_htc_file, **kwargs):
        self.base_htc_file = base_htc_file
        for k, v in kwargs.items():
            setattr(self, k, v)

    def from_pandas(self, dataFrame, write_input_files=True):
        if not isinstance(dataFrame, PandasObject) and hasattr(dataFrame, 'to_pandas'):
            dataFrame = dataFrame.to_pandas()
        if isinstance(dataFrame, pd.Panel):
            return {n: self.from_pandas(df.dropna(how='all'), write_input_files)for n, df in dataFrame.iteritems()}

        else:
            assert 'Name' in dataFrame
            if write_input_files:
                for _, row in dataFrame.iterrows():
                    self.write_input_files(**row.to_dict())
        return dataFrame

    def from_excel(self, excel_file, write_input_files=True):
        return self.from_pandas(pd.read_excel(excel_file), write_input_files)

    def from_CVF(self, constants, variables, functions, write_input_files=True):
        attributes = list(constants.keys()) + list(variables.keys()) + list(functions.keys())
        tags = []
        for var in itertools.product(*list(variables.values())):
            this_dict = dict(constants)
            var_dict = dict(zip(variables.keys(), var))
            this_dict.update(var_dict)
            for key, func in functions.items():
                this_dict[key] = func(this_dict)

            tags.append(list(this_dict.values()))

        df = pd.DataFrame(tags, columns=attributes)
        return self.from_pandas(df, write_input_files)

    def from_definition(self, definition_file, write_input_files=True):
        file = os.path.splitext(definition_file)[0]
        module = os.path.relpath(file, os.getcwd()).replace(os.path.sep, ".")
        def_mod = importlib.import_module(module)
        return self.from_CVF(def_mod.constants, def_mod.variables, def_mod.functions, write_input_files)

    def __call__(self, name, folder='', **kwargs):
        return self.write_input_files(name, folder, **kwargs)

    def write_input_files(self, Name, Folder='', **kwargs):
        htc = HTCFile(self.base_htc_file)
        for k, v in kwargs.items():
            k = k.replace('/', '.')
            if '.' in k:
                line = htc[k]
                v = str(v).strip().replace(",", " ")
                line.values = v.split()
            else:
                getattr(self, 'set_%s' % k)(htc, **kwargs)
        htc.set_name(Name, Folder)
        htc.save()
        args = {'Name': Name, 'Folder': Folder}
        args.update(kwargs)

        return pd.Series(args)


def main():
    if __name__ == '__main__':

        path = os.path.dirname(test_files.__file__) + '/simulation_setup/DTU10MWRef6.0/'
        base_htc_file = path + 'htc/DTU_10MW_RWT.htc'
        h2writer = HAWC2InputWriter(base_htc_file)

        # pandas2htc
        df = pd.DataFrame({'wind.wsp': [4, 6, 8], 'Name': ['c1', 'c2', 'c3']})
        h2writer.from_pandas(df)

        # HAWC2InputWriter
        class MyWriter(HAWC2InputWriter):
            def set_time(self, htc, time, **_):
                htc.set_time(self.time_start, self.time_start + time)

        myWriter = MyWriter(base_htc_file, time_start=100)
        myWriter("t1", 'tmp', time=600, **{"wind.wsp": 10})

        # from_CVF (constants, variables and functions)
        constants = {'simulation.time_stop': 100}
        variables = {'wind.wsp': [4, 6, 8],
                     'wind.tint': [0.1, 0.15, 0.2]}
        functions = {'Name': lambda x: 'sim_wsp' + str(x['wind.wsp']) + '_ti' + str(x['wind.tint'])}
        df = h2writer.from_CVF(constants, variables, functions, write_input_files=False)
        print(df)


main()



# -*- coding: utf-8 -*-
"""
Created on Mon Aug  8 10:22:49 2016

@author: dave
"""
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import
from builtins import dict
from io import open
from builtins import zip
from builtins import range
from builtins import str
from builtins import int
from future import standard_library
standard_library.install_aliases()
from builtins import object

# standard python library
import os
import zipfile
import copy
import tarfile
import glob
import shutil
import tempfile

import numpy as np
import pandas as pd
#from tqdm import tqdm

from wetb.prepost.Simulations import Cases


def create_chunks_htc_pbs(cases, sort_by_values=['[Windspeed]'], ppn=20, i0=0,
                          nr_procs_series=9, queue='workq', compress=False,
                          walltime='24:00:00', chunks_dir='zip-chunks-jess',
                          wine_arch='win32', wine_prefix='~/.wine32',
                          pyenv_cmd='source /home/python/miniconda3/bin/activate',
                          pyenv='wetb_py3'):
    """Group a large number of simulations htc and pbs launch scripts into
    different zip files so we can run them with find+xargs on various nodes.
    """

    def chunker(seq, size):
        # for DataFrames you can also use groupby, as taken from:
        # http://stackoverflow.com/a/25703030/3156685
        # for k,g in df.groupby(np.arange(len(df))//10)
        # but this approach is faster, see also:
        # http://stackoverflow.com/a/25701576/3156685
        # http://stackoverflow.com/a/434328/3156685
        return (seq[pos:pos + size] for pos in range(0, len(seq), size))

    def make_zip_chunks(df, ii, sim_id, run_dir, model_zip):
        """Create zip cunks and also create an index
        """

        # create a new zip file, give index of the first element. THis is
        # quasi random due to the sorting we applied earlier
#        ii = df.index[0]
        rpl = (sim_id, ii)
        fname = os.path.join(run_dir, chunks_dir, '%s_chnk_%05i' % rpl)
        zf = zipfile.ZipFile(fname+'.zip', 'w', compression=zipfile.ZIP_STORED)

        # start with appending the base model zip file
        fname_model = os.path.join(run_dir, model_zip)
        with zipfile.ZipFile(fname_model, 'r') as zf_model:
            for n in zf_model.namelist():
                # FIXME: might be duplicates when creating dirs from dirtags
                zf.writestr(n, zf_model.open(n).read())

        # create all necessary directories in the zip file
        dirtags = ['[htc_dir]', '[res_dir]','[log_dir]','[animation_dir]',
                   '[pbs_in_dir]', '[eigenfreq_dir]','[turb_dir]','[micro_dir]',
                   '[meander_dir]','[hydro_dir]', '[mooring_dir]',
                   '[pbs_in_dir]', '[pbs_out_dir]']
        dirnames = []
        for tag in dirtags:
            for dirname in set(df[tag].unique().tolist()):
                if not dirname or dirname.lower() not in ['false', 'none', 0]:
                    dirnames.append(dirname)
        for dirname in set(dirnames):
            if dirname != 0:
                # FIXME: might have duplicates from the base model zip file
                zf.write('.', os.path.join(dirname, '.'))

        # and the post-processing data
        # FIXME: do not use hard coded paths!
        zf.write('.', 'prepost-data/')

        # HTC files
        df_src = df['[run_dir]'] + df['[htc_dir]'] + df['[case_id]']
        df_dst = df['[htc_dir]'] + df['[case_id]']
        # create an index so given the htc file, we can find the chunk nr
        df_index = pd.DataFrame(index=df['[case_id]'].copy(),
                                columns=['chnk_nr', 'name'])
        df_index['chnk_nr'] = ii
        df_index['name'] = os.path.join(chunks_dir, '%s_chnk_%05i' % rpl)
        # Since df_src and df_dst are already Series, iterating is fast an it
        # is slower to first convert to a list
        for src, dst_rel in zip(df_src, df_dst):
            zf.write(src+'.htc', dst_rel+'.htc')

        # PBS files
        df_src = df['[run_dir]'] + df['[pbs_in_dir]'] + df['[case_id]']
        df_dst = df['[pbs_in_dir]'] + df['[case_id]']
        # Since df_src and df_dst are already Series, iterating is fast an it
        # is slower to first convert to a list
        for src, dst_rel in zip(df_src, df_dst):
            zf.write(src+'.p', dst_rel+'.p')

        # copy and rename input files with given versioned name to the
        # all files that will have to be renamed to their non-changeable
        # default file name.
        # this is a bit more tricky since unique() will not work on list items
        copyto_files_tmp = df['[copyto_files]'].astype(str)
        copyto_files = []
        # cycle through the unique elements
        for k in set(copyto_files_tmp):
            # k is of form: "['some/file.txt', 'another/file1.txt']"
            if len(k) < 2:
                continue
            items = [kk[1:-1] for kk in k.split('[')[1].split(']')[0].split(', ')]
            copyto_files.extend(items)
        # we might still have non unique elements
        copyto_files = set(copyto_files)
        for copyto_file, dst_rel in zip(copyto_files, df_dst):
            src = os.path.join(run_dir, copyto_file)
            # make dir if it does not exist
            zf.write('.', os.path.dirname(copyto_file), '.')
            zf.write(src, copyto_file)

        zf.close()

        return fname, df_index

    pbs_tmplate = "\n"
    pbs_tmplate += "### Standard Output\n"
    pbs_tmplate += "#PBS -N [job_name]\n"
    pbs_tmplate += "#PBS -o [std_out]\n"
    pbs_tmplate += "### Standard Error\n"
    pbs_tmplate += "#PBS -e [std_err]\n"
    pbs_tmplate += "#PBS -W umask=[umask]\n"
    pbs_tmplate += "### Maximum wallclock time format HOURS:MINUTES:SECONDS\n"
    pbs_tmplate += "#PBS -l walltime=[walltime]\n"
    pbs_tmplate += "#PBS -l nodes=[nodes]:ppn=[ppn]\n"
    pbs_tmplate += "### Queue name\n"
    pbs_tmplate += "#PBS -q [queue]\n"
    pbs_tmplate += "\n"

    # FIXME: this causes troubles on CI runner for the tests (line endings?)
#     pbs_tmplate = """
# ### Standard Output
# #PBS -N [job_name]
# #PBS -o [std_out]
# ### Standard Error
# #PBS -e [std_err]
# #PBS -W umask=[umask]
# ### Maximum wallclock time format HOURS:MINUTES:SECONDS
# #PBS -l walltime=[walltime]
# #PBS -l nodes=[nodes]:ppn=[ppn]
# ### Queue name
# #PBS -q [queue]

# """

    def make_pbs_chunks(df, ii, sim_id, run_dir, model_zip, compress=False,
                        wine_arch='win32', wine_prefix='~/.wine32'):
        """Create a PBS that:
            * copies all required files (zip chunk) to scratch disk
            * copies all required turbulence files to scratch disk
            * runs everything with find+xargs
            * copies back what's need to mimer
            """
#        ii = df.index[0]
        cmd_find = '/home/MET/sysalt/bin/find'
        cmd_xargs = '/home/MET/sysalt/bin/xargs'
        jobid = '%s_chnk_%05i' % (sim_id, ii)

        wineparam = (wine_arch, wine_prefix)

        pbase = os.path.join('/scratch','$USER', '$PBS_JOBID', '')
        post_dir_base = post_dir.split(sim_id)[1]
        if post_dir_base[0] == os.path.sep:
            post_dir_base = post_dir_base[1:]

        # FIXME: commonpath was only added in Python 3.5, but CI runner is old
        try:
            compath = os.path.commonpath
        except AttributeError:
            compath = os.path.commonprefix

        pbs_in_base = compath(df['[pbs_in_dir]'].unique().tolist())
        pbs_in_base = os.path.join(pbs_in_base, '')
        htc_base = compath(df['[htc_dir]'].unique().tolist())
        htc_base = os.path.join(htc_base, '')
        res_base = compath(df['[res_dir]'].unique().tolist())
        res_base = os.path.join(res_base, '')
        log_base = compath(df['[log_dir]'].unique().tolist())
        log_base = os.path.join(log_base, '')

        # =====================================================================
        # PBS HEADER
        pbs = copy.copy(pbs_tmplate)
        pbs = pbs.replace('[job_name]', jobid)
        pbs = pbs.replace('[std_out]', './pbs_out_chunks/%s.out' % jobid)
        pbs = pbs.replace('[std_err]', './pbs_out_chunks/%s.err' % jobid)
        pbs = pbs.replace('[umask]', '0003')
        pbs = pbs.replace('[walltime]', walltime)
        pbs = pbs.replace('[nodes]', str(nodes))
        pbs = pbs.replace('[ppn]', str(ppn))
        pbs = pbs.replace('[queue]', queue)
        pbs += '\necho "%s"\n' % ('-'*70)

        # =====================================================================
        # activate the python environment
        if pyenv is not None:
            pbs += 'echo "activate python environment %s"\n' % pyenv
            rpl = (pyenv_cmd, pyenv)
            pbs += '%s %s\n' % rpl
            # sometimes activating an environment fails due to a FileExistsError
            # is this because it is activated at the same time on another node?
            # check twice if the environment got activated for real,
            # but only do so for /home/python/miniconda
            pbs += 'echo "CHECK 2x IF %s IS ACTIVE, IF NOT TRY AGAIN"\n' % pyenv
            pbs += 'CMD=\"from distutils.sysconfig import get_python_lib;'
            pbs += 'print (get_python_lib().find(\'/usr/lib/python\'))"\n'
            pbs += 'ACTIVATED=`python -c "$CMD"`\n'
            pbs += 'if [ $ACTIVATED -eq 0 ]; then %s %s;fi\n' % rpl
            pbs += 'ACTIVATED=`python -c "$CMD"`\n'
            pbs += 'if [ $ACTIVATED -eq 0 ]; then %s %s;fi\n' % rpl

        # =====================================================================
        # create all necessary directories at CPU_NR dirs
        # browse to scratch directory
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += 'cd %s\n' % pbase
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n\n'
        pbs += 'echo "create CPU directories on the scratch disk"\n'
        pbs += 'mkdir -p %s\n' % os.path.join(pbase, sim_id, '')
        for k in range(ppn):
            pbs += 'mkdir -p %s\n' % os.path.join(pbase, '%i' % k, '')

        # =====================================================================
        # get the zip-chunk file from the PBS_O_WORKDIR
        pbs += '\n'
        pbs += 'echo "%s"\n' % ('-'*70)
        pbs += 'cd $PBS_O_WORKDIR\n'
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "get the zip-chunk file from the PBS_O_WORKDIR"\n'
        # copy the relevant zip chunk file to the scratch main directory
        rpl = (os.path.join('./', chunks_dir, jobid), os.path.join(pbase, ''))
        pbs += 'cp %s.zip %s\n' % rpl

        # =====================================================================
        # unzip to all cpu dirs
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += 'cd %s\n' % pbase
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "unzip chunk, create dirs in cpu and sim_id folders"\n'
        # unzip chunk, this contains all relevant folders already, and also
        # contains files defined in [copyto_files]
        for k in list(range(ppn)) + [sim_id]:
            dst = os.path.join('%s' % k, '.')
            pbs += '/usr/bin/unzip %s -d %s >> /dev/null\n' % (jobid+'.zip', dst)

        # =====================================================================
        # create all turb_db directories
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += 'cd %s\n' % os.path.join(pbase, sim_id, '')
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "create turb_db directories"\n'
        turb_db_tags = ['[turb_db_dir]', '[meander_db_dir]', '[micro_db_dir]']
        turb_db_dirs = []
        for tag in turb_db_tags:
            for dirname in set(df[tag].unique().tolist()):
                dirname_s = str(dirname).replace('.', '').replace('/', '')
                if dirname_s.lower() not in ['false', 'none', '0']:
                    turb_db_dirs.append(dirname)
        turb_db_dirs = set(turb_db_dirs)
        # create all turb dirs
        for dirname in sorted(turb_db_dirs):
            pbs += 'mkdir -p %s\n' % os.path.join(dirname, '')

        # =====================================================================
        # copy required turbulence from db_dir to scratch/db_dirs
        # turb_db_dir might not be set, same for turb_base_name, for those
        # cases we do not need to copy anything from the database to the node
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += 'cd $PBS_O_WORKDIR\n'
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        base_name_tags = ['[turb_base_name]', '[meander_base_name]',
                          '[micro_base_name]']
        for db, base_name in zip(turb_db_tags, base_name_tags):
            turb_db_dirs = df[db] + df[base_name]
            # When set to None, the DataFrame will have text as None
            # FIXME: CI runner has and old pandas version (v0.14.1)
            try:
                p1 = turb_db_dirs.str.find('None')==-1
                p2 = turb_db_dirs.str.find('none')==-1
                p3 = turb_db_dirs.str.find('false')==-1
                p4 = turb_db_dirs.str.find('False')==-1
#                p4 = turb_db_dirs.str.find('0')==-1
                turb_db_src = turb_db_dirs[p1 & p2 & p3 & p4]
            except AttributeError:
                # and findall returns list with the search str occuring as
                # many times as found in the str...
                # sel should be True if str does NOT occur in turb_db_dirs
                # meaning if findall returns empty list
                sel = [True]*len(turb_db_dirs)
                for val in ['false', 'none', 'None', 'False']:
                    findall = turb_db_dirs.str.findall(val).tolist()
                    # len==0 if nothing has been found
                    sel_ = [True if len(k)==0 else False for k in findall]
                    # merge with other search results, none of the elements
                    # should occur
                    sel = [True if k and kk else False for k, kk in zip(sel, sel_)]
                turb_db_src = turb_db_dirs[sel]
            pbs += '\n'
            pbs += '# copy to scratch db directory for %s, %s\n' % (db, base_name)
            for k in sorted(turb_db_src.unique()):
                dst = os.path.dirname(os.path.join(pbase, sim_id, k))
                pbs += 'cp %s* %s\n' % (k, os.path.join(dst, '.'))

        # =====================================================================
        # to be safe, create all turb dirs in the cpu dirs
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += 'cd %s\n' % os.path.join(pbase, '')
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "create turb directories in CPU dirs"\n'
        turb_dir_tags = ['[turb_dir]', '[meander_dir]', '[micro_dir]']
        turb_dirs = []
        for tag in turb_dir_tags:
            for dirname in set(df[tag].unique().tolist()):
                dirname_s = str(dirname).replace('.', '').replace('/', '')
                if dirname_s.lower() not in ['false', 'none', '0']:
                    turb_dirs.append(dirname)
        turb_dirs = sorted(set(turb_dirs))
        for k in list(range(ppn)):
            for dirname in turb_dirs:
                pbs += 'mkdir -p %s\n' % os.path.join(str(k), dirname, '')

        # =====================================================================
        # symlink everything from the turb_db_dir to the cpu/turb_dir
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += 'cd %s\n' % os.path.join(pbase, sim_id, '')
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "Link all turb files into CPU dirs"\n'
        for db_dir, turb_dir in zip(turb_db_tags, turb_dir_tags):
            # FIXME: this needs to be written nicer. We should be able to
            # select from df non-defined values so we can exclude them
            # now it seems they are either None, False or 0 in either
            # boolean, str or int formats
            nogo = ['false', 'none', '0']
            try:
                symlink_dirs = df[db_dir] + '_*_'
                symlink_dirs = symlink_dirs + df[turb_dir]
            except:
                continue
            for symlink in symlink_dirs.unique().tolist():
                db_dir, turb_dir = symlink.split('_*_')
                db_dir_s = db_dir.replace('.', '').replace('/', '').lower()
                turb_dir_s = turb_dir.replace('.', '').replace('/', '').lower()
                if db_dir_s in nogo or turb_dir_s in nogo:
                    continue
                db_dir_abs = os.path.join(pbase, sim_id, db_dir, '')
                for k in list(range(ppn)):
                    turb_dir_abs = os.path.join(pbase, str(k), turb_dir, '')
                    rpl = (db_dir_abs, turb_dir_abs)
                    pbs += 'find %s -iname "*.bin" -exec ln -s {} %s \\;\n' % rpl

        # copy all from scratch/turb_db to cpu/turb
        # turb_dir_base = os.path.join(compath(list(turb_dirs)), '')
        # pbs += '\necho "%s"\n' % ('-'*70)
        # pbs += 'cd %s\n' % os.path.join(pbase, sim_id, '')
        # pbs += "echo 'current working directory:'\n"
        # pbs += 'pwd\n'
        # pbs += 'echo "Link all turb files into CPU dirs"\n'
        # for k in range(ppn):
        #     rpl = (os.path.relpath(os.path.join(sim_id, turb_dir_base)), k)
        #     pbs += 'find %s -iname "*.bin" -exec cp {} %s/{} \\;\n' % rpl

        # =====================================================================
        # finally we can run find+xargs!!!
        pbs += '\n'
        pbs += 'echo "%s"\n' % ('-'*70)
        pbs += 'cd %s\n' % pbase
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "START RUNNING JOBS IN find+xargs MODE"\n'
        pbs += 'WINEARCH=%s WINEPREFIX=%s winefix\n' % wineparam
        pbs += '# run all the PBS *.p files in find+xargs mode\n'
        pbs += 'echo "following cases will be run from following path:"\n'
        pbs += 'echo "%s"\n' % (os.path.join(sim_id, pbs_in_base))
        pbs += 'export LAUNCH_PBS_MODE=false\n'
        rpl = (cmd_find, os.path.join(sim_id, pbs_in_base))
        pbs += "%s %s -type f -name '*.p' | sort -z\n" % rpl
        pbs += '\n'
        pbs += 'echo "number of files to be launched: "'
        pbs += '`find %s -type f | wc -l`\n' % os.path.join(sim_id, pbs_in_base)
        rpl = (cmd_find, os.path.join(sim_id, pbs_in_base), cmd_xargs, ppn)
        cmd = ("%s %s -type f -name '*.p' -print0 | sort -z | %s -0 -I{} "
               "--process-slot-var=CPU_NR -n 1 -P %i sh {}\n" % rpl)
        pbs += cmd
        pbs += 'echo "END OF JOBS IN find+xargs MODE"\n'

        # =====================================================================
        # move results back from the node sim_id dir to the origin
        pbs += '\n'
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += "echo 'total scratch disk usage:'\n"
        pbs += 'du -hs %s\n' % pbase
        pbs += 'cd %s\n' % os.path.join(pbase, sim_id)
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "Results saved at sim_id directory:"\n'
        rpl = (os.path.join(pbs_in_base, '*'), os.path.join(htc_base, '*'))
        pbs += 'find .\n'

        if compress:
            # compress all result files into an archive, first *.sel files
            # FIXME: why doesn this work with -name "*.sel" -o -name "*.dat"??
            pbs += '\necho "move results into compressed archive"\n'
            pbs += 'find %s -name "*.sel" -print0 ' % res_base
            fname = os.path.join(res_base, 'resfiles_chnk_%05i' % ii)
            pbs += '| xargs -0 tar --remove-files -rf %s.tar\n' % fname
            # now add the *.dat files to the archive
            pbs += 'find %s -name "*.dat" -print0 ' % res_base
            fname = os.path.join(res_base, 'resfiles_chnk_%05i' % ii)
            pbs += '| xargs -0 tar --remove-files -rf %s.tar\n' % fname
            pbs += 'xz -z2 -T %i %s.tar\n' % (ppn, fname)

            # compress all logfiles into an archive
            pbs += '\necho "move logfiles into compressed archive"\n'
            pbs += 'find %s -name "*.log" -print0 ' % log_base
            fname = os.path.join(log_base, 'logfiles_chnk_%05i' % ii)
            pbs += '| xargs -0 tar --remove-files -rf %s.tar\n' % fname
            pbs += 'xz -z2 -T %i %s.tar\n' % (ppn, fname)

        # compress all post-processing results (saved as csv's) into an archive
        pbs += '\necho "move statsdel into compressed archive"\n'
        pbs += 'find %s -name "*.csv" -print0 ' % res_base
        fname = os.path.join(post_dir_base, 'statsdel_chnk_%05i' % ii)
        pbs += '| xargs -0 tar --remove-files -rf %s.tar\n' % fname
        pbs += 'xz -z2 -T %i %s.tar\n' % (ppn, fname)

        # compress all post-processing results (saved as csv's) into an archive
        pbs += '\necho "move log analysis into compressed archive"\n'
        pbs += 'find %s -name "*.csv" -print0 ' % log_base
        fname = os.path.join(post_dir_base, 'loganalysis_chnk_%05i' % ii)
        pbs += '| xargs -0 tar --remove-files -rf %s.tar\n' % fname
        pbs += 'xz -z2 -T %i %s.tar\n' % (ppn, fname)

        pbs += '\n'
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += 'cd %s\n' % pbase
        pbs += "echo 'current working directory:'\n"
        pbs += 'pwd\n'
        pbs += 'echo "move results back from node scratch/sim_id to origin, '
        pbs += 'but ignore htc, and pbs_in directories."\n'

        tmp = os.path.join(sim_id, '*')
        pbs += 'echo "copy from %s to $PBS_O_WORKDIR/"\n' % tmp
        pbs += 'time rsync -au --remove-source-files %s $PBS_O_WORKDIR/ \\\n' % tmp
        pbs += '    --exclude %s \\\n' % os.path.join(pbs_in_base, '*')
        pbs += '    --exclude *.htc\n'
        # when using -u, htc and pbs_in files should be ignored
#        pbs += 'time cp -ru %s $PBS_O_WORKDIR/\n' % tmp
        if pyenv is not None:
            pbs += 'source deactivate\n'
        pbs += 'echo "DONE !!"\n'
        pbs += '\necho "%s"\n' % ('-'*70)
        pbs += '# in case wine has crashed, kill any remaining wine servers\n'
        pbs += '# caution: ALL the users wineservers will die on this node!\n'
        pbs += 'echo "following wineservers are still running:"\n'
        pbs += 'ps -u $USER -U $USER | grep wineserver\n'
        pbs += 'killall -u $USER wineserver\n'
        pbs += 'exit\n'

        rpl = (sim_id, ii)
        fname = os.path.join(run_dir, chunks_dir, '%s_chnk_%05i' % rpl)
        with open(fname+'.p', 'w') as f:
            f.write(pbs)

    def make_pbs_postpro_chunks():
        """When only the post-processing has to be re-done for a chunk.
        """
        pass


    cc = Cases(cases)
    df = cc.cases2df()
    # sort on the specified values in the given columns
    # FIXME: sort_values was only added in Pandas 0.17, but CI runner is old
    try:
        df.sort_values(by=sort_by_values, inplace=True)
    except AttributeError:
        df.sort(columns=sort_by_values, inplace=True)

    # create the directory to store all zipped chunks
    try:
        os.mkdir(os.path.join(df['[run_dir]'].iloc[0], chunks_dir))
    # FIXME: how do you make this work pythonically on both PY2 and PY3?
    except (FileExistsError, OSError):
        pass

    fpath = os.path.join(df['[run_dir]'].iloc[0], 'pbs_out_chunks')
    if not os.path.exists(fpath):
        os.makedirs(fpath)

    # remove all cases that start with test_ and move them into a separate
    # chunk
    # FIXME: CI runner has and old pandas version (v0.14.1)
    try:
        sel_notest = df['[Case folder]'].str.find('test_')<0
        sel_test = ~sel_notest
    except AttributeError:
        # and findall returns list with the search str occuring as
        # many times as found in the str...
        findall = df['[Case folder]'].str.findall('test_').tolist()
        # len==0 if nothing has been found
        sel_notest = [True if len(k)==0 else False for k in findall]
        sel_test = [not k for k in sel_notest]
    df_dlc = df[sel_notest]
    df_test = df[sel_test]

    # DLC CHUNKS
    df_dlc_iter = chunker(df_dlc, nr_procs_series*ppn)
    sim_id = df['[sim_id]'].iloc[0]
    run_dir = df['[run_dir]'].iloc[0]
    model_zip = df['[model_zip]'].iloc[0]
    post_dir = df['[post_dir]'].iloc[0]
    nodes = 1
    df_ind = pd.DataFrame(columns=['chnk_nr'], dtype=np.int32)
    df_ind.index.name = '[case_id]'
    for ii, dfi in enumerate(df_dlc_iter):
        fname, ind = make_zip_chunks(dfi, i0+ii, sim_id, run_dir, model_zip)
        make_pbs_chunks(dfi, i0+ii, sim_id, run_dir, model_zip,
                        wine_arch=wine_arch, wine_prefix=wine_prefix,
                        compress=compress)
        df_ind = df_ind.append(ind)
        print(fname)
    fname = os.path.join(post_dir, 'case_id-chunk-index')
    df_ind['chnk_nr'] = df_ind['chnk_nr'].astype(np.int32)
    df_ind.to_hdf(fname+'.h5', 'table', compression=9, complib='zlib')
    df_ind.to_csv(fname+'.csv')

    # TEST CHUNKS
    df_test_iter = chunker(df_test, nr_procs_series*ppn)
    df_ind = pd.DataFrame(columns=['chnk_nr'], dtype=np.int32)
    df_ind.index.name = '[case_id]'
    for ii, dfi in enumerate(df_test_iter):
        fname, ind = make_zip_chunks(dfi, 90000+ii, sim_id, run_dir, model_zip)
        make_pbs_chunks(dfi, 90000+ii, sim_id, run_dir, model_zip,
                        wine_arch=wine_arch, wine_prefix=wine_prefix,
                        compress=compress)
        df_ind = df_ind.append(ind)
        print(fname)
    fname = os.path.join(post_dir, 'case_id-chunk-test-index')
    df_ind['chnk_nr'] = df_ind['chnk_nr'].astype(np.int32)
    df_ind.to_hdf(fname+'.h5', 'table', compression=9, complib='zlib')
    df_ind.to_csv(fname+'.csv')


def regroup_tarfiles(cc):
    """Re-group all chunks again per [Case folder] compressed file. First all
    chunks are copied to the node scratch disc, then start working on them.
    This only works on a node with PBS stuff.

    Make sure to maintain the same location as defined by the tags!

    [res_dir] and [Case folder] could be multiple directories deep, bu the
    final archive will only contain the files (no directory structure), and
    the name of the archive is that of the last directory:
        /[res_dir]/[Case folder]/[Case folder].tar.xz
        /res/dir/case/folder/dlcname/dlcname.tar.xz

    Parameters
    ----------

    path_pattern : str
        /path/to/files/*.tar.xz

    """

    USER = os.getenv('USER')
    PBS_JOBID = os.getenv('PBS_JOBID')
    scratch = os.path.join('/scratch', USER, PBS_JOBID)
    src = os.getenv('PBS_O_WORKDIR')

    path_pattern = '/home/dave/SimResults/NREL5MW/D0022/prepost-data/*.xz'

    for ffname in tqdm(glob.glob(path_pattern)):
        appendix = os.path.basename(ffname).split('_')[0]
        with tarfile.open(ffname, mode='r:xz') as tar:
            # create new tar files if necessary for each [Case folder]
            for tarinfo in tar.getmembers():
                t2_name = os.path.basename(os.path.dirname(tarinfo.name))
                t2_dir = os.path.join(os.path.dirname(path_pattern), t2_name)
                if not os.path.isdir(t2_dir):
                    os.makedirs(t2_dir)
                t2_path = os.path.join(t2_dir, t2_name + '_%s.tar' % appendix)
                fileobj = tar.extractfile(tarinfo)
                # change the location of the file in the new archive:
                # the location of the archive is according to the folder
                # structure as defined in the tags, remove any subfolders
                tarinfo.name = os.basename(tarinfo.name)
                with tarfile.open(t2_path, mode='a') as t2:
                    t2.addfile(tarinfo, fileobj)


def merge_from_tarfiles(df_fname, path, pattern, tarmode='r:xz', tqdm=False,
                        header='infer', names=None, sep=',', min_itemsize={},
                        verbose=False, dtypes={}):
    """Merge all csv files from various tar archives into a big pd.DataFrame
    store.

    Parameters
    ----------

    df_fname : str
        file name of the pd.DataFrame h5 store in which all chunks will be
        merged. Names usually used are:
            * [sim_id]_ErrorLogs.h5
            * [sim_id]_statistics.h5

    path : str
        Directory in which all chunks are located.

    pattern : str
        Search pattern used to select (using glob.glob) files in path

    tarmode : str, default='r:xz'
        File opening mode for tarfile (used when opening each of the chunks).

    tqdm : boolean, default=False
       If True, an interactive progress bar will be displayed (requires the
       tqdm module). If set to False no progress bar will be displayed.

    header : int, default='infer'
        Argument passed on to pandas.read_csv. Default to 'infer', set to
        None if there is no header, set to 0 if header is on first row.

    names : list of column names, default=None
        Argument passed on to pandas.read_csv. Default to None. List with
        column names to be used in the DataFrame.

    min_itemsize : dict, default={}
        Argument passed on to pandas.HDFStore.append. Set the minimum lenght
        for a given column in the DataFrame.

    sep : str, default=','
        Argument passed on to pandas.read_csv. Set to ';' when handling the
        ErrorLogs.

    """

    store = pd.HDFStore(os.path.join(path, df_fname), mode='w', format='table',
                        complevel=9, complib='zlib')

    if tqdm:
        from tqdm import tqdm
    else:
        def tqdm(itereable):
            return itereable

    for tar_fname in tqdm(glob.glob(os.path.join(path, pattern))):
        if verbose:
            print(tar_fname)
        with tarfile.open(tar_fname, mode=tarmode) as tar:
            df = pd.DataFrame()
            for tarinfo in tar.getmembers():
                fileobj = tar.extractfile(tarinfo)
                tmp = pd.read_csv(fileobj, header=header, names=names, sep=sep)
                for col, dtype in dtypes.items():
                    tmp[col] = tmp[col].astype(dtype)
                df = df.append(tmp)
            try:
                if verbose:
                    print('writing...')
                store.append('table', df, min_itemsize=min_itemsize)
            except Exception as e:
                if verbose:
                    print('store columns:')
                    print(store.select('table', start=0, stop=0).columns)
                    print('columns of the DataFrame being added:')
                    print(df.columns)
                storecols = store.select('table', start=0, stop=0).columns
                store.close()
                print(e)
                return df, storecols

    store.close()


# TODO: make this class more general so you can also just give a list of files
# to be merged, excluding the tar archives.
class AppendDataFrames(object):
    """Merge DataFrames, either in h5 or csv format, located in (compressed)
    tar archives.
    """

    def __init__(self, tqdm=False):
        if tqdm:
            from tqdm import tqdm
        else:
            def tqdm(itereable):
                return itereable
        self.tqdm = tqdm

    def _open(self, fname, tarmode='r:xz'):
        """Open text file directly or from a tar archive. Return iterable
        since a tar archive might contain several csv text files
        """

        if fname.find('.tar') > -1:
            with tarfile.open(fname, mode=tarmode) as tar:
                for tarinfo in tar.getmembers():
                    linesb = tar.extractfile(tarinfo).readlines()
                    # convert from bytes to strings
                    lines = [line.decode() for line in linesb]
                    yield lines, tarinfo.name
        else:
            with open(fname, 'r') as f:
                lines = f.readlines()
            yield lines, os.path.basename(fname)

    def df2store(self, store, path, tarmode='r:xz', min_itemsize={},
                 colnames=None, header='infer', columns=None, sep=';',
                 index2col=None, ignore_index=True, fname_col=False):
        """This is very slow, use txt2txt instead.
        """

        # TODO: it seems that with threading you could parallelize this kind
        # of work: http://stackoverflow.com/q/23598063/3156685
        # http://stackoverflow.com/questions/23598063/
        # multithreaded-web-scraper-to-store-values-to-pandas-dataframe

        # http://gouthamanbalaraman.com/blog/distributed-processing-pandas.html

        df = pd.DataFrame()
        for fname in self.tqdm(glob.glob(path)):
            with tarfile.open(fname, mode=tarmode) as tar:
                df = pd.DataFrame()
                for tarinfo in tar.getmembers():
                    fileobj = tar.extractfile(tarinfo)
                    if tarinfo.name[-2:] == 'h5':
                        tmp = pd.read_hdf(fileobj, 'table', columns=columns)
                    elif tarinfo.name[-3:] == 'csv':
                        tmp = pd.read_csv(fileobj, sep=sep, names=colnames,
                                          header=header, usecols=columns)
                    else:
                        continue
                    if index2col is not None:
                        # if the index does not have a name we can still set it
                        tmp[index2col] = tmp.index
                        tmp[index2col] = tmp[index2col].astype(str)
                        tmp.reset_index(level=0, drop=True, inplace=True)
                    # add the file name as a column
                    if fname_col:
                        case_id = os.path.basename(tarinfo.name)
                        tmp[fname_col] = '.'.join(case_id.split('.')[:-1])
                        tmp[fname_col] = tmp[fname_col].astype(str)
                    df = df.append(tmp, ignore_index=ignore_index)

                store.append('table', df, min_itemsize=min_itemsize)
#                if len(df) > w_every:
#                    # and merge into the big ass DataFrame
#                    store.append('table', df, min_itemsize=min_itemsize)
#                    df = pd.DataFrame()
        return store

    # FIXME: when merging log file analysis (files with header), we are still
    # skipping over one case
    def txt2txt(self, fjoined, path, tarmode='r:xz', header=None, sep=';',
                fname_col=False, header_fjoined=None, recursive=False):
        """Read as strings, write to another file as strings.

        Parameters
        ----------

        fjoined

        path

        tarmode

        header : int, default=None
            Indicate if data files contain a header and on which line it is
            located. Set to None if data files do not contain header, and in
            that case the joined file will not contain a header either. All
            lines above the header are ignored.

        sep

        fname_col

        header_fjoined : str, default=None
            If the data files do not contain a header write out header_fjoined
            as the header of the joined file.

        recursive

        """
        if isinstance(header, int):
            write_header = True
            icut = header + 1
        else:
            # when header is None, there is no header
            icut = 0
            write_header = False
        if isinstance(header_fjoined, str):
            write_header = True

        with tempfile.NamedTemporaryFile(mode='w', delete=False) as ft:
            ftname = ft.name
            for fname in self.tqdm(glob.glob(path, recursive=recursive)):
                for lines, case_id in self._open(fname, tarmode=tarmode):
                    # only include the header at the first round
                    if write_header:
                        if header_fjoined is None:
                            header_fjoined = lines[header]
                        # add extra column with the file name if applicable
                        if fname_col:
                            rpl = sep + fname_col + '\n'
                            header_fjoined = header_fjoined.replace('\n', rpl)
                        ft.write(header_fjoined)
                        write_header = False
                    # but cut out the header on all other occurances
                    case_id = '.'.join(case_id.split('.')[:-1])
                    for line in lines[icut:]:
                        if fname_col:
                            line = line.replace('\n', sep + case_id + '\n')
                        ft.write(line)
                ft.flush()

        # and move from temp dir to fjoined
        shutil.move(ftname, fjoined)

    def csv2df_chunks(self, store, fcsv, chunksize=100000, min_itemsize={},
                      colnames=None, dtypes={}, header='infer', sep=';'):
        """Convert a large csv file to a pandas.DataFrame in chunks using
        a pandas.HDFStore.
        """
        df_iter = pd.read_csv(fcsv, chunksize=chunksize, sep=sep,
                              names=colnames, header=header)
        for df_chunk in self.tqdm(df_iter):
            for col, dtype in dtypes.items():
                df_chunk[col] = df_chunk[col].astype(dtype)
            store.append('table', df_chunk, min_itemsize=min_itemsize)
        return store


if __name__ == '__main__':
    pass




import sys
import time

from wetb.utils.ui import OutputUI, InputUI, StatusUI


class TextOutputUI(OutputUI):
    last_text = ""
    
    def __init__(self, parent=None):
        OutputUI.__init__(self, parent=parent)
        self.errors = sys.stdout.errors
        self.encoding = sys.stdout.encoding
        sys.stdout = self
        sys.stderr = self
        
        
        
    def show_message(self, msg, title="Information"):
        #self.last_text = msg
        if title != "":
            print ("\n\n%s\n%s\n%s\n" % (title, "-"*len(title), msg))
        else:
            print (msg)

    def show_warning(self, msg, title="Warning"):
        #self.last_text = msg
        print ("%s\n%s\n%s" % (title, "-"*len(title), msg))

    def show_text(self, text, end="\n", flush=False):
        #self.last_text = text
        #print (text, end=end)
        self.write(text+end)
        if flush:
            sys.stdout.flush()
            
    def flush(self):
        sys.__stdout__.flush()
            
    def write(self, txt):
        if txt.strip()!="":
            self.last_text = txt
        sys.__stdout__.write(txt)
        

class TextInputUI(InputUI):
    def get_confirmation(self, title, msg):
        raise NotImplementedError

    def get_string(self, title, msg):
        raise NotImplementedError

    def get_open_filename(self, title="Open", filetype_filter="*.*", file_dir=None, selected_filter=None):
        raise NotImplementedError
    def get_save_filename(self, title, filetype_filter, file_dir=None, selected_filter=None):
        raise NotImplementedError


    def get_open_filenames(self, title, filetype_filter, file_dir=None):
        raise NotImplementedError

    def get_foldername(self, title='Select directory', file_dir=None):
        raise NotImplementedError



class TextStatusUI(StatusUI, TextOutputUI):
    

    def progress_iterator(self, sequence, text="Working... Please wait", allow_cancel=True, always_refresh=False):
        global pct
        it = iter(list(sequence))
        if it.__length_hint__() > 0:
            def init():
                global pct
                self.show_text("\n\n|0" + " " * 46 + "50%" + " " * 46 + "100%|")
                pct = 0
                self.show_text("|", end="")

            self.show_text(text, flush=True)
            #sys.__stdout__.flush()
            N = it.__length_hint__()
            init()
            for n, v in enumerate(it):
                #if n % 100 == 99:
                #    self.show_text("")
                if (self.last_text != "." and n > 0) or (always_refresh and ((n + 1) / N * 100 > pct)):
                    init()
                while ((n + 1) / N * 100 > pct):
                    self.show_text(".", end="", flush=True)
                    pct += 1
                yield(v)
            self.show_text("|")


    def exec_long_task(self, text, allow_cancel, task, *args, **kwargs):
        print (text)
        return task(*args, **kwargs)

    def start_wait(self):
        #print ("Working please wait")
        pass

    def end_wait(self):
        #print ("finish")
        pass


    def progress_callback(self, text="Working... Please wait", always_refresh=False):
        class ProgressCallBack():
            
            def __init__(self, ui, text, always_refresh=False):
                self.ui = ui
                self.text = text
                self.always_refresh = always_refresh
                self.pct = None
            
            def init(self):
                self.ui.show_text("\n\n" + self.text, flush=True)
                self.ui.show_text("|0" + " " * 46 + "50%" + " " * 46 + "100%|")
                self.pct = 0
                self.ui.show_text("|", end="")
                
            def __call__(self, n, N):
                if (self.pct is None) or (self.ui.last_text != "." and self.pct > 0) or (self.always_refresh and ((n + 1) / N * 100 > self.pct)):
                    self.init()
                while ((n + 1) / N * 100 > (self.pct+1)):
                    self.ui.show_text(".", end="", flush=True)
                    self.pct += 1
                if self.pct==100:
                    self.ui.show_text("|")
        return ProgressCallBack(self, text, always_refresh)
                
class TextUI(TextInputUI, TextStatusUI):
    pass


if __name__ == "__main__":
    def task(callback):
        for i in range(100):
            callback(i,100)
            time.sleep(0.05)
            
    task (TextStatusUI().progress_callback())
        


import numpy as np


def standard_power_curve(power_norm, diameter, turbulence_intensity=.1, shear=(0, 100),
                         rho=1.225, max_cp=.49,
                         gear_loss_const=.01, gear_loss_var=.014, generator_loss=0.03, converter_loss=.03):
    """Generate standard power curve

    The method is extracted from an Excel spreadsheet made by Kenneth Thomsen, DTU Windenergy and
    ported into python by Mads M. Pedersen, DTU Windenergy.

    Parameters
    ----------
    power_norm : int or float
        Nominal power [kW]
    diameter : int or float
        Rotor diameter [m]
    turbulence_intensity : float
        Turbulence intensity [%]
    shear : (power shear coefficient, hub height)
        Power shear arguments\n
        - Power shear coeficient, alpha\n
        - Hub height [m]
    rho : float optional
        Density of air [kg/m^3], defualt is 1.225
    max_cp : float
        Maximum power coefficient
    gear_loss_const : float
        Constant gear loss [%]
    gear_loss_var : float
        Variable gear loss [%]
    generator_loss : float
        Generator loss [%]
    converter_loss : float

    Examples
    --------
    wsp, power = standard_power_curve(10000, 178.3)
    plot(wsp, power)
    show()
    """


    area = (diameter / 2) ** 2 * np.pi
    wsp_lst = np.arange(0.5, 25, .5)
    sigma_lst = wsp_lst * turbulence_intensity

    def norm_dist(x, my, sigma):
        if turbulence_intensity > 0:
            return 1 / np.sqrt(2 * np.pi * sigma ** 2) * np.exp(-(x - my) ** 2 / (2 * sigma ** 2))
        else:
            return x == my
    p_aero = .5 * rho * area * wsp_lst ** 3 * max_cp / 1000

    # calc power - gear, generator and conv loss
#     gear_loss = gear_loss_const * power_norm + gear_loss_var * p_aero
#     p_gear = p_aero - gear_loss
#     p_gear[p_gear < 0] = 0
#     gen_loss = generator_loss * p_gear
#     p_gen = p_gear - gen_loss
#     converter_loss = converter_loss * p_gen
#     p_raw = p_gen - converter_loss
#     p_raw[p_raw > power_norm] = power_norm
    p_raw = p_aero - power_loss(p_aero, power_norm, gear_loss_const, gear_loss_var, generator_loss, converter_loss)

    powers = []
    shear_weighted_wsp = []
    alpha, hub_height = shear
    r = diameter / 2
    z = np.linspace(hub_height - r, hub_height + r, 100, endpoint=True)
    shear_factors = (z / hub_height) ** alpha
    rotor_width = 2 * np.sqrt(r ** 2 - (hub_height - z) ** 2)

    for wsp, sigma in zip(wsp_lst, sigma_lst):
        shear_weighted_wsp.append(wsp * (np.trapz(shear_factors ** 3 * rotor_width, z) / (area)) ** (1 / 3))
        ndist = norm_dist(wsp_lst, wsp, sigma)
        powers.append((ndist * p_raw).sum() / ndist.sum())

    return wsp_lst, lambda wsp : np.interp(wsp, wsp_lst, powers)
    return wsp_lst, np.interp(shear_weighted_wsp, wsp_lst, powers)

def power_loss(power_aero, power_norm, gear_loss_const=.01, gear_loss_var=.014, generator_loss=0.03, converter_loss=.03):
    gear_loss = gear_loss_const * power_norm + gear_loss_var * power_aero
    p_gear = power_aero - gear_loss
    p_gear[p_gear < 0] = 0
    gen_loss = generator_loss * p_gear
    p_gen = p_gear - gen_loss
    converter_loss = converter_loss * p_gen
    p_electric = p_gen - converter_loss
    p_electric[p_electric > power_norm] = power_norm
    return power_aero - p_electric
    

if __name__ == '__main__':


    from matplotlib.pyplot import plot, show
    plot(*standard_power_curve(10000, 178.3, 0., (0, 119), gear_loss_const=.0, gear_loss_var=.0, generator_loss=0.0, converter_loss=.0))
    plot(*standard_power_curve(10000, 178.3, 0.03, (0, 119), gear_loss_const=.0, gear_loss_var=.0, generator_loss=0.0, converter_loss=.0))




    show()



'''
Created on 24/04/2014

@author: MMPE
'''
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from io import open
from builtins import range
from future import standard_library
standard_library.install_aliases()

import numpy as np
class AtTimeFile(object):
    """Loads an at time file generated by HAWC2
    
    Functions are generated for each sensor in the file from the template:
    xxx(l=None, curved_length=False)
    
    E.g.
    twist(36, curved_length=True) # Twist 36m from the root along the (curved) center line
    twist(36) # Twist at radius 36m (Straight line radius, nondim. with bladetip_radius which must be specified in constructor)
    twist() # Twist at all aerodynamic calculation points
        
    
    >>> at = AtTimeFile("at.dat", 86.3655) # load file
    >>> at.attribute_names # Attribute names
    ['radius_s', 'twist', 'chord']
    >>> at[:3,1]) # first 3 twist rows
    [-14.5    -14.5002 -14.5007]
    >>> at.twist()[:3]) # Twist first 3 twist rows
    [-14.5    -14.5002 -14.5007]
    >>> at.twist(36, curved_length=True) # Twist at curved_length = 36 (interpolated)
    -5.172195702789108
    >>> at.twist(36) # Twist at radius = 36 (interpolated)
    -5.162084567646019
    """
    def __init__(self, filename, bladetip_radius=None):
        """
        Parameters
        ----------
        filename : string
            Filename
        bladetip_radius : int, float or None, optional
            Radius of blade tip. Used to convert from curved length to radius  
        """
        self.blade_radius = bladetip_radius
        with open(filename, encoding='utf-8') as fid:
            lines = fid.readlines()
        atttribute_name_line = [l.strip().startswith("# Radius_s") for l in lines].index(True)
        #self.attribute_names = lines[atttribute_name_line].lower().replace("#", "").split()
        self.attribute_names = [n.strip() for n in lines[atttribute_name_line].lower().split("#")[1:]]
        data = np.array([[float(l) for l in lines[i].split() ] for i in range(atttribute_name_line+1, len(lines))])
        self.data = data
        def func_factory(column):
            def values(l=None, curved_length=False):
                assert curved_length==True or curved_length==False, "Curved length must be boolean, but is %s"%curved_length
                if l is None:
                    return self.data[:, column]
                else:
                    if curved_length:
                        return np.interp(l, self.data[:, 0], self.data[:, column])
                    else:
                        assert self.blade_radius is not None, "bladetip_radius must be specified in __init__ when requesting value of radius (alternatively you can request for curved_length)"
                        return np.interp(l/self.blade_radius, self.data[:, 0]/self.data[-1, 0], self.data[:, column])
                
            return values
        for column, att_name in enumerate(self.attribute_names):
            setattr(self, att_name, func_factory(column))

    def radius_curved_ac(self, radius=None):
        """Radius (curved distance) of aerodynamic calculation point(s) 

        Parameters
        ----------
        radius : int or float, optional
            - if None (default): Radius of calculation points\n
            - if int or float: Radius of calculation point nearest radius

        Returns
        -------
        radius : float or array_like
            Radius of calculation points or radius of calculation point nearest radius
        """
        if radius is None:
            return self.radius_s()
        else:
            return self.radius_s()[np.argmin(np.abs(self.radius_s() - radius))]

    def value(self, radius, column):
        return np.interp(radius, self.data[:, 0], self.data[:, column])

    def __getitem__(self, subset):
        return self.data[subset]


if __name__ == "__main__":
    at = AtTimeFile(r"tests/test_files/at.dat", 86.3655) # load file
    at = AtTimeFile(r'U:\hama\HAWC2-AVATAR\res/avatar-7ntm-scaled-rad.dat')
    
    at.attribute_names # Attribute names
    at[:3,1] # first 3 twist rows
    print (len(at.attribute_names))
    print ("\n".join(at.attribute_names))
    print (at.data.shape)
    #at.twist()[:3] # Twist first 3 twist rows
    #print (at.twist(36, curved_length=True)) # Twist at curved_length = 36 (interpolated)
    #print (at.twist(36)) # Twist at 36 (interpolated)
    




'''
Created on 17/07/2014

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
import unittest
from wetb.hawc2.at_time_file import AtTimeFile
import numpy as np
import os


tfp = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path
class TestAtTimeFile(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.testfilepath = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path


    def test_doc_examples(self):
        atfile = AtTimeFile(self.testfilepath + "at_time.dat", bladetip_radius=20.501)  # load file
        self.assertEqual(atfile.attribute_names, ['radius_s', 'twist', 'chord'])
        np.testing.assert_array_equal(atfile[:3, 1], [ 0., -0.775186, -2.91652 ])
        np.testing.assert_array_equal(atfile.twist()[:3], [ 0. , -0.775186 , -2.91652 ])
        self.assertAlmostEqual(atfile.twist(10), -5.34743208242399)  # Twist at radius = 10 (interpolated)
        self.assertEqual(atfile.twist(10,curved_length=True), -5.34743208242399)  # Twist at radius = 10 (interpolated)



    def test_at_time_file(self):
        atfile = AtTimeFile(self.testfilepath + "at_time.dat")
        self.assertEqual(atfile.attribute_names, ['radius_s', 'twist', 'chord'])
        self.assertEqual(atfile.radius_s()[9], 6.32780)
        self.assertEqual(atfile.twist()[9], -13.5373)
        self.assertEqual(atfile.chord()[9], 1.54999)


    def test_at_time_file_at_curved_radius(self):
        atfile = AtTimeFile(self.testfilepath + "at_time.dat")
        self.assertEqual(atfile.radius_s(9, curved_length=True), 9)
        self.assertEqual(atfile.twist(9, curved_length=True), -6.635983309665461)
        self.assertEqual(atfile.chord(9, curved_length=True), 1.3888996578373045)

    def test_at_time_file_at_radius(self):
        atfile = AtTimeFile(self.testfilepath + "at_time.dat", bladetip_radius=20.501/2)
        self.assertEqual(atfile.radius_s(9/2), 9)
        self.assertEqual(atfile.twist(9/2), -6.635983309665461)
        self.assertEqual(atfile.chord(9/2), 1.3888996578373045)


    def test_at_time_file_radius(self):
        atfile = AtTimeFile(self.testfilepath + "at_time.dat")
        self.assertEqual(atfile.radius_curved_ac()[12], 10.2505)
        self.assertEqual(atfile.radius_curved_ac(10), 10.2505)
        self.assertEqual(atfile.radius_curved_ac(10.5), 10.2505)

    def test_rotor_name_null(self):
        atfile = AtTimeFile(self.testfilepath + "at_time/test_rotor_name_null.dat", bladetip_radius=20.501)  # load file


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 17/07/2014

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
import unittest
from wetb.hawc2.cmp_test_cases import CompareTestCases
import numpy as np
import os

class TestCmpTestCases(CompareTestCases):

    def setUp(self):
        unittest.TestCase.setUp(self)
        tfp = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path
        self.ref_path = tfp + 'cmp_test_cases/ref/'
        self.test_path = tfp + 'cmp_test_cases/test1/'

    def test_compare_sel(self):
        self.compare_sel(self.ref_path + 'test1.sel', self.test_path + 'test1.sel')

#    def test_compare_sel_different_number_of_lines(self):
#        self.assertRaises(AssertionError, self.compare_sel, self.ref_path + 'test2.sel', self.test_path + 'test2.sel')

    def test_compare_sel_different_header(self):
        self.compare_sel(self.ref_path + 'test3.sel', self.test_path + 'test3.sel')

    def test_compare_sel_difference(self):
        self.assertRaises(AssertionError,# "     2      bea1 angle                     deg        shaft_rot angle",
                          self.compare_sel, self.ref_path + 'test4.sel',
                          self.test_path + 'test4.sel')

    def test_compare_contents_dat(self):
        self.compare_dat_contents(self.ref_path + 'test1.dat', self.test_path + 'test1.dat')

    def test_compare_dat_contents_difference(self):
        # FIXME: can we have something similar in PY2 that checks the error
        # text as well?
        self.assertRaises(AssertionError,# "  2.00000E-02",
                          self.compare_dat_contents, self.ref_path + 'test3.dat',
                          self.test_path + 'test3.dat')

    #def test_compare_plot_difference(self):
    #    self.assertRaises(AssertionError, "Difference in the the values of:\n1 Time", self.compare_dat_plot, self.ref_path + 'test3', self.test_path + 'test3')

#    def test_compare_folder(self):
#        self.compare_folder(r'test_files/ref/', r'test_files/test/', 'ref', 'test1')


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()




'''
Created on 20. jul. 2017

@author: mmpe
'''
import os
import wetb
import inspect
from urllib.request import urlretrieve

wetb_rep_path = os.path.abspath(os.path.dirname(wetb.__file__) + "/../") + "/"
local_TestFiles_path = wetb_rep_path + "TestFiles/"                                   
remote_TestFiles_url = "https://gitlab.windenergy.dtu.dk/toolbox/TestFiles/raw/master/"


def _absolute_filename(filename):
    if not os.path.isabs(filename):
        index = [os.path.realpath(s[1]) for s in inspect.stack()].index(__file__) + 2
        caller_module_path = os.path.dirname(inspect.stack()[index][1])
        tfp = caller_module_path + "/test_files/"
        filename = tfp + filename
    return filename

def get_test_file(filename):
    filename = _absolute_filename(filename) 
    if not os.path.exists(filename):
        rel_path = os.path.relpath(filename, wetb_rep_path).replace("\\","/")
        filename = local_TestFiles_path + rel_path
        if not os.path.exists(filename):
            urlretrieve(remote_TestFiles_url + rel_path, filename)
    return filename
        
        



def move2test_files(filename):
    filename = _absolute_filename(filename)
    assert os.path.isfile(filename), filename
    dst_filename = os.path.join(wetb_rep_path, 'TestFiles', os.path.relpath(filename, wetb_rep_path))
    folder = os.path.dirname(dst_filename)
    if not os.path.exists(folder):
        os.makedirs(folder)
    os.rename(filename, dst_filename)
    
    
    
    
    


'''
Created on 17/07/2014

@author: MMPE
'''
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
from wetb.hawc2.ae_file import AEFile
standard_library.install_aliases()
import os
import unittest
from wetb.hawc2.pc_file import PCFile


import numpy as np



class TestPCFile(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.testfilepath = os.path.join(os.path.dirname(__file__), 'test_files/')  # test file path

    def test_PCFile_ae(self):
        pc = PCFile(self.testfilepath + "NREL_5MW_pc.txt")
        ae = AEFile(self.testfilepath + "NREL_5MW_ae.txt")
        thickness = ae.thickness(36)
        self.assertEqual(pc.CL(thickness, 10), 1.358)
        self.assertEqual(pc.CD(thickness, 10), 0.0255)
        self.assertEqual(pc.CM(thickness, 10), -0.1103)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on 08/02/2016

@author: mmpe
'''

import numpy as np




def saturated_vapor_pressure(T):
    """Calculate pressure of saturated water vapor at specified temperature as described at
    http://wahiduddin.net/calc/density_altitude.htm
    Parameters
    ---------
    t : float
        Temperature [C]

    Returns
    -------
    float
        Pressure [mb]
    """

    eso = 6.1078
    c0 = 0.99999683
    c1 = -0.90826951 * 10 ** -2
    c2 = 0.78736169 * 10 ** -4
    c3 = -0.61117958 * 10 ** -6
    c4 = 0.43884187 * 10 ** -8
    c5 = -0.29883885 * 10 ** -10
    c6 = 0.21874425 * 10 ** -12
    c7 = -0.17892321 * 10 ** -14
    c8 = 0.11112018 * 10 ** -16
    c9 = -0.30994571 * 10 ** -19
    p = (c0 + T * (c1 + T * (c2 + T * (c3 + T * (c4 + T * (c5 + T * (c6 + T * (c7 + T * (c8 + T * (c9))))))))))
    return eso / p ** 8

def saturated_vapor_pressure2(t):
    """Calculate pressure of saturated water vapor at specified temperature as described at
    http://wahiduddin.net/calc/density_altitude.htm
    Parameters
    ---------
    t : float
        Temperature [C]

    Returns
    -------
    float
        Pressure [mb]
    """
    c0 = 6.1078
    c1 = 7.5
    c2 = 237.3
    return c0 * 10 ** ((c1 * t) / (c2 + t))

def saturated_vapor_pressure3(t):
    """Calculate pressure of saturated water vapor at specified temperature according to

    The IAPWS Formulation 1995 for the Thermodynamic Properties of Ordinary Water Substance for General and Scientific Use
    W. Wagner and A. Pruss
    J. Phys. Chem. Ref. Data 31, 387 (2002); http://dx.doi.org/10.1063/1.1461829

    Parameters
    ---------
    t : float
        Temperature [C]

    Returns
    -------
    float
        Pressure [mb]
    """

    T = t + 273.15
    Tc = 647.096
    Pc = 220640
    v = 1 - T / Tc
    C1 = -7.85951783
    C2 = 1.84408259
    C3 = -11.7866497
    C4 = 22.6807411
    C5 = -15.9618719
    C6 = 1.80122502
    return Pc * np.exp(Tc / T * (C1 * v + C2 * v ** 1.5 + C3 * v ** 3 + C4 * v ** 3.5 + C5 * v ** 4 + C6 * v ** 7.5))

def saturated_vapor_pressure4(t):

    """Calculate the saturated vapor pressure as described in
    http://www.vaisala.com/Vaisala%20Documents/Application%20notes/Humidity_Conversion_Formulas_B210973EN-F.pdf

    Parameters
    ---------
    t : float
        Temperature [C]

    Returns
    -------
    Saturated vapor pressure [mBar]
    """
    A = 6.116441
    m = 7.591386
    Tn = 240.7263
    return A * 10 ** ((m * t) / (t + Tn))

def saturated_vapor_pressure_IEC(t):
    """Calculate the saturated vapor pressure according to IEC 61400-12-1

    Parameters
    ---------
    t : float
        Temperature [C]

    Returns
    -------
    Saturated vapor pressure [mBar]
    """

    T = t + 273.15
    return 0.000000205 * np.exp(0.0631846 * T)



def drew_point(t, RH):
    A = 6.116441
    m = 7.591386
    Tn = 240.7263
    Pw = saturated_vapor_pressure4(t) * RH / 100
    return Tn / (m / np.log10(Pw / A) - 1)

def air_density(P, t, rh=0, saturated_vapor_pressure_function=saturated_vapor_pressure):
    """Calculate the density of atmospheric air at specified pressure, temperature and humidity
    source: http://wahiduddin.net/calc/density_altitude.htm
    Equivalent to formulation in IEC61400-12-1 if used with the saturated_vapor_pressure_IEC function


    Parameters
    ---------
    P : float
        Atmospheric pressure [mb]=[hPa]
    t : float
        Temperature [C]
    rh : float
        Relative humidity [%]
    saturated_vapor_pressure_function : function
        Function, f(t)->P, that takes the temperature in celcius as input and
        returns the saturated vapor pressure in mbar

    Returns
    -------
    float
        Density of atmospheric air
    """
    Pv = saturated_vapor_pressure_function(t) * rh / 100
    Pd = P - Pv
    Rv = 461.4964
    Rd = 287.0531
    Tk = t + 273.15
    return (Pd * 100 / (Rd * Tk)) + (Pv * 100 / (Rv * Tk))

def R(rh=0, t=15, P=1014):
    """Specific gas constant ~287.058 J/(kg K) for dry air
    
    Parameters
    ---------
    rh : float
        Relative humidity [%]
    t : float
        Temperature [C]
    P : float
        pressure [hPa]
    
    Returns
    -------
    Specific gas constant
    """
    #assert np.all((900<P)&(P<1100)), "Pressure outside range 900 to 1100"
    #assert np.all((-50<t)&(t<100)), "Temperature outside range -50 to 100"
    Tk = t + 273.15
    return P * 100 / (air_density(P, t, rh) * Tk)

if __name__ == "__main__":
    pass
    
#     #print (air_density(1013, 65, 50))
#     #print (drew_point(40, 50))
#     import matplotlib.pyplot as plt
#     if 0:
#         t = np.arange(-20, 50)
#         plt.plot(t, saturated_vapor_pressure(t))
#         plt.plot(t, saturated_vapor_pressure2(t), "--")
#         plt.plot(t, saturated_vapor_pressure3(t), ":")
#         plt.show()
# 
#     if 0:
#         t = np.arange(5, 25)
#         plt.xlabel("Temperature [C]")
#         plt.ylabel("Density [kg/m3]")
#         plt.plot(t, air_density(990, t, 50), label='P: 990 hPa')
#         plt.plot(t, air_density(1018, t, 50), label='P: 1018 hPa')
#         plt.legend()
#         plt.show()
# 
#     p = 1013
#     t = 20
#     rh = 70
#     print (R(p, t, rh) * (t + 273.15))
#     print ((p * 100) / air_density(p, t, rh))
#     
#     print (R(p,10,80))
#     print (R(p,100,80))
# 
#     if 0:
#         rh = np.arange(0, 100)
#         plt.xlabel("Rh [%]")
#         plt.ylabel("R")
#         plt.plot(rh, R(990, 20, rh), label='P: 990 hPa, 20 t')
# 
#         plt.legend()
#         plt.show()



'''
Created on 15/01/2014

@author: MMPE
'''
import unittest

import numpy as np
from wetb.utils.geometry import rad
from wetb.utils.rotation import transformation_matrix, mdot, dots, rotate, rotate_x, \
    rotmat, rotate_y, rotate_z

x, y, z = 0, 1, 2
class Test(unittest.TestCase):


    def test_transformation_matrix(self):
        np.testing.assert_array_almost_equal(transformation_matrix(rad(30), x), [[[ 1., 0. , 0.],
                                                                                  [ 0., 0.8660254, 0.5],
                                                                                  [ 0., -0.5 , 0.8660254]]])
        np.testing.assert_array_almost_equal(transformation_matrix(rad(30), y), [[[ 0.8660254, 0. , -0.5],
                                                                     [ 0., 1., 0 ],
                                                                     [ 0.5, 0        , 0.8660254]]])
        np.testing.assert_array_almost_equal(transformation_matrix(rad(30), z), [[[ 0.8660254, 0.5 , 0.],
                                                                                  [ -0.5, 0.8660254, 0],
                                                                                  [ 0., 0 , 1]]])

    def test_rotation_matrix(self):
        np.testing.assert_array_almost_equal(rotmat(rad(30), x), [[[ 1., 0. , 0.],
                                                                   [ 0., 0.8660254, -0.5],
                                                                   [ 0., 0.5 , 0.8660254]]])
        np.testing.assert_array_almost_equal(rotmat(rad(30), y), [[[ 0.8660254, 0. , 0.5],
                                                                   [ 0., 1., 0 ],
                                                                   [ -0.5, 0        , 0.8660254]]])
        np.testing.assert_array_almost_equal(rotmat(rad(30), z), [[[ 0.8660254, -0.5 , 0.],
                                                                   [ 0.5, 0.8660254, 0],
                                                                   [ 0., 0 , 1]]])
    def test_rotation_matrixes(self):
        np.testing.assert_array_almost_equal(rotmat([rad(30), rad(60)], 0), [[[ 1., 0. , 0.],
                                                                     [ 0., 0.8660254, -0.5],
                                                                     [ 0., 0.5 , 0.8660254]],
                                                                             [[ 1., 0. , 0.],
                                                                     [ 0., 0.5, -0.8660254, ],
                                                                     [ 0., +0.8660254, 0.5 ]]])

    def test_mdot(self):
        x, y, z = 0, 1, 2
        m1 = transformation_matrix(np.pi, x)
        m2 = transformation_matrix([np.pi, np.pi / 2], y)

        np.testing.assert_array_almost_equal(m1, [[[1, 0, 0], [0, -1, 0], [0, 0, -1]]])
        np.testing.assert_array_almost_equal(mdot(m1, m1), [[[1, 0, 0], [0, 1, 0], [0, 0, 1]]])
        np.testing.assert_array_almost_equal(mdot(m1, m2), [[[-1, 0, 0], [0, -1, 0], [0, 0, 1]], [[0, 0, -1], [0, -1, 0], [-1, 0, 0]]])
        np.testing.assert_array_almost_equal(mdot(m2, m2), [[[1, 0, 0], [0, 1, 0], [0, 0, 1]], [[-1, 0, 0], [0, 1, 0], [0, 0, -1]]])
        np.testing.assert_array_almost_equal(mdot(m2, m1), [[[-1, 0, 0], [0, -1, 0], [0, 0, 1]], [[0, 0, 1], [0, -1, 0], [1, 0, 0]]])


    def test_dots(self):
        x, y, z = 0, 1, 2
        v1 = np.array([1, 2, 3])
        v2 = np.array([1, 2, 4])

        np.testing.assert_array_almost_equal(dots(transformation_matrix(-np.pi / 2, x), v1).T, [[1, -3, 2]])
        np.testing.assert_array_almost_equal(dots(transformation_matrix(-np.pi , x), v1).T, [[1, -2, -3]])

        np.testing.assert_array_almost_equal(dots(transformation_matrix(-np.pi / 2, y), v1).T, [[3, 2, -1]])
        np.testing.assert_array_almost_equal(dots(transformation_matrix(-np.pi , y), v1).T, [[-1, 2, -3]])

        np.testing.assert_array_almost_equal(dots(transformation_matrix(-np.pi / 2, z), v1).T, [[-2, 1, 3]])
        np.testing.assert_array_almost_equal(dots(transformation_matrix(-np.pi , z), v1).T, [[-1, -2, 3]])

        v = np.array([v1, v2]).T

        np.testing.assert_array_almost_equal(dots(transformation_matrix([-np.pi / 2, np.pi], x), v).T, [[1, -3, 2], [1, -2, -4]])
        np.testing.assert_array_almost_equal(dots(transformation_matrix([-np.pi / 2, np.pi], y), v).T, [[3, 2, -1], [-1, 2, -4]])
        np.testing.assert_array_almost_equal(dots(transformation_matrix([-np.pi / 2, np.pi], z), v).T, [[-2, 1, 3], [-1, -2, 4]])
        np.testing.assert_array_almost_equal(dots([transformation_matrix(np.pi / 2, z), transformation_matrix(np.pi / 2, y), transformation_matrix(np.pi / 2, x)], v1).T, [[3, -2, 1]])


    def test_rotate(self):
        x, y, z = 0, 1, 2
        v = np.array([1, 2, 3])

        np.testing.assert_array_almost_equal(rotate(rotmat(np.pi / 2, x), v), [1, -3, 2])
        np.testing.assert_array_almost_equal(rotate(rotmat(np.pi, x), v), [1, -2, -3])

        np.testing.assert_array_almost_equal(rotate(rotmat(np.pi / 2, y), v), [3, 2, -1])
        np.testing.assert_array_almost_equal(rotate(rotmat(np.pi, y), v), [-1, 2, -3])

        np.testing.assert_array_almost_equal(rotate(rotmat(np.pi / 2, z), v), [-2, 1, 3])
        np.testing.assert_array_almost_equal(rotate(rotmat(np.pi, z), v), [-1, -2, 3])

        v = np.array([[1, 2, 3], [1, 2, 4]])

        np.testing.assert_array_almost_equal(rotate(rotmat([np.pi / 2, -np.pi], x), v)[0], [1, -3, 2])
        np.testing.assert_array_almost_equal(rotate(rotmat([np.pi / 2, -np.pi], x), v)[1], [1, -2, -4])

        np.testing.assert_array_almost_equal(rotate(rotmat([np.pi / 2, np.pi], y), v)[0], [3, 2, -1])
        np.testing.assert_array_almost_equal(rotate(rotmat([np.pi / 2, np.pi], y), v)[1], [-1, 2, -4])

        np.testing.assert_array_almost_equal(rotate(rotmat([np.pi / 2, np.pi], z), v)[0], [-2, 1, 3])
        np.testing.assert_array_almost_equal(rotate(rotmat([np.pi / 2, np.pi], z), v)[1], [-1, -2, 4])


    def test_rotate_xyz(self):
        x, y, z = 0, 1, 2
        v = np.array([1, 2, 3])

        np.testing.assert_array_almost_equal(rotate_x(v, np.pi / 2), [1, -3, 2])
        np.testing.assert_array_almost_equal(rotate_x(v, -np.pi), [1, -2, -3])

        np.testing.assert_array_almost_equal(rotate_y(v, np.pi / 2), [3, 2, -1])
        np.testing.assert_array_almost_equal(rotate_y(v, np.pi), [-1, 2, -3])

        np.testing.assert_array_almost_equal(rotate_z(v, np.pi / 2), [-2, 1, 3])
        np.testing.assert_array_almost_equal(rotate_z(v, np.pi), [-1, -2, 3])

        v = np.array([[1, 2, 3], [4, 5, 6]])
        np.testing.assert_array_almost_equal(rotate_z(v, np.pi / 2), [[-2, 1, 3], [-5, 4, 6]])
        np.testing.assert_array_almost_equal(rotate_z(v, [np.pi / 2]), [[-2, 1, 3], [-5, 4, 6]])
        np.testing.assert_array_almost_equal(rotate_z(v, [-np.pi / 2, np.pi]), [[2, -1, 3], [-4, -5, 6]])


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_rad']
    unittest.main()



import numpy as np
import warnings

def psd(data, sample_frq=1., no_segments=1, bin_size=1):
    """Map a signal to frequency domain (Power spectrum density)

    Parameters
    ----------
    data : array_like
        The time domain signal
    sample_frq : float, optional
        The sample frequency of the signal. Affects the frequency part of the result only\n
        Default is 1.
    no_segments : int, optional
        Remove noise by dividing the signal into no_segments bins and average the PSD of the subsets.\n
        As the time span of the bins are shorter, low frequencies are sacrificed.\n
        If e.g. 2, the signal is split into two parts and the result is the mean
        of the PSD for the first and the last part of the signal\n
        Default is 1

        Be carefull when using many segments, as the result may be corrupted as the function does not use any window function.
    bin_size : int, optional
        Smoothen the PSD by dividing the frequency list and PSD values into bins of size 'bin_size' and return the means of the bins

    Returns
    -------
    f : array_like
        Frequencies
    psd : array_like
        Power spectrum density

    Examples
    --------
    >>> ds = OpenHawc2("scripts/test.sel")
    >>> f, psd = PSD(ds(3)[:],ds.sample_frq, 2)
    >>> Plot(yscale='log')
    >>> PlotData(None, f, psd)
    """
    if no_segments is None:
        no_segments = 1

    nfft = data.shape[-1] // no_segments
    data = data[:no_segments * nfft].reshape(no_segments, nfft)


    NumUniquePts = int(np.ceil((nfft + 1) / 2))
    f = np.linspace(0, sample_frq / 2., NumUniquePts)
    fftx = np.abs(np.fft.rfft(data))

    fftx = fftx.sum(0)
    y = fftx / (nfft * no_segments)

    y = y ** 2
    y *= 2  # Because negative half is ignored

    y /= f[1] - f[0]  # distribute discrete energy values to continuous areas


    f, y = f[1:], y[1:]  # Remove 0Hz frequency contribution
    bins = (NumUniquePts - 1) // bin_size
    def smoothen(x, N):
        M = (x.shape[0] - N + 1) // N
        return np.array([x[i:M * N + i].reshape(M, N).mean(1) for i in range(N)]).T.flatten()
    if bin_size > 1:
        f, y = smoothen(f, bin_size), smoothen(y, bin_size)
#    with warnings.catch_warnings():
#        warnings.simplefilter("ignore", category=RuntimeWarning)  # ignore runtime warning of nanmean([nan])
#        f = np.nanmean(f[:bins * bin_size].reshape(bins, f.shape[0] // bins), 1)
#        y = np.nanmean(y[:bins * bin_size].reshape(bins, y.shape[0] // bins), 1)
    return (f, y)



'''
Created on 20/01/2014

@author: MMPE

See documentation of HTCFile below

'''
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from builtins import zip
from builtins import int
from builtins import str
from future import standard_library
import os
standard_library.install_aliases()
from collections import OrderedDict
import collections


class OrderedDict(collections.OrderedDict):
    pass

    def __str__(self):
        return "\n".join(["%-30s\t %s" % ((str(k) + ":"), str(v)) for k, v in self.items()])


def parse_next_line(lines):
    _3to2list = list(lines.pop(0).split(";"))
    line, comments, = _3to2list[:1] + [_3to2list[1:]]
    comments = ";".join(comments).rstrip()
    while lines and lines[0].lstrip().startswith(";"):
        comments += "\n%s" % lines.pop(0).rstrip()
    return line.strip(), comments


def fmt_value(v):
    try:
        if int(float(v)) == float(v):
            return int(float(v))
        return float(v)
    except ValueError:
        return v.replace("\\", "/")


c = 0


class HTCContents(object):
    lines = []
    contents = None
    name_ = ""
    parent = None

    def __setitem__(self, key, value):
        if isinstance(key, str):
            self.contents[key] = value
        elif isinstance(key, int):
            self.values[key] = value
        else:
            raise NotImplementedError
        value.parent = self

    def __getitem__(self, key):
        if isinstance(key, str):
            key = key.replace(".", "/")
            if "/" in key:
                keys = key.split('/')
                val = self.contents[keys[0]]
                for k in keys[1:]:
                    val = val[k]
                return val
            return self.contents[key]
        else:
            return self.values[key]

    def __getattr__(self, *args, **kwargs):
        if args[0] in ['__members__', '__methods__']:
            # fix python2 related issue. In py2, dir(self) calls
            # __getattr__(('__members__',)), and this call must fail unhandled to work
            return object.__getattribute__(self, *args, **kwargs)
        try:
            return object.__getattribute__(self, *args, **kwargs)
        except:
            k = args[0]
            if k.endswith("__1"):
                k = k[:-3]
            return self.contents[k]

    def __setattr__(self, *args, **kwargs):
        _3to2list1 = list(args)
        k, v, = _3to2list1[:1] + _3to2list1[1:]
        if k in dir(self):  # in ['section', 'filename', 'lines']:
            return object.__setattr__(self, *args, **kwargs)
        if isinstance(v, str):
            v = [fmt_value(v) for v in v.split()]
        if isinstance(v, HTCContents):
            self.contents[k] = v
        else:
            if not isinstance(v, (list, tuple)):
                v = [v]
            self.contents[k] = HTCLine(k, v, "")

    def __delattr__(self, *args, **kwargs):
        k, = args
        if k in self:
            del self.contents[k]

    def __iter__(self):
        return iter(self.contents.values())

    def __contains__(self, key):
        if self.contents is None:
            return False
        return key in self.contents

    def get(self, section, default=None):
        try:
            return self[section]
        except KeyError:
            return default

    def keys(self):
        return list(self.contents.keys())

    def _add_contents(self, contents):
        if contents.name_ not in self:
            self[contents.name_] = contents
        else:
            ending = "__2"
            while contents.name_ + ending in self:
                ending = "__%d" % (1 + float("0%s" % ending.replace("__", "")))
            self[contents.name_ + ending] = contents

    def add_section(self, name, allow_duplicate=False):
        if name in self and allow_duplicate is False:
            return self[name]
        if name == "output":
            section = HTCOutputSection(name)
        elif name.startswith("output_at_time"):
            section = HTCOutputAtTimeSection(name)
        else:
            section = HTCSection(name)
        self._add_contents(section)
        return section

    def add_line(self, name, values, comments=""):
        line = HTCLine(name, values, comments)
        self._add_contents(line)
        return line

    def location(self):
        if self.parent is None:
            return os.path.basename(self.filename)
        else:
            return self.parent.location() + "/" + self.name_

    def compare(self, other):
        my_keys = self.keys()
        other_keys = other.keys()
        s = ""
        while my_keys or other_keys:
            if my_keys:
                if (my_keys[0] in other_keys):
                    while other_keys[0] != my_keys[0]:
                        s += "\n".join(["+ %s" % l for l in str(other[other_keys.pop(0)]).strip().split("\n")]) + "\n\n"

                    s += self[my_keys.pop(0)].compare(other[other_keys.pop(0)])

                else:
                    s += "\n".join(["- %s" % l for l in str(self[my_keys.pop(0)]).strip().split("\n")]) + "\n\n"
            else:
                s += "\n".join(["+ %s" % l for l in str(other[other_keys.pop(0)]).strip().split("\n")]) + "\n\n"
        return s


class HTCSection(HTCContents):
    end_comments = ""
    begin_comments = ""

    def __init__(self, name, begin_comments="", end_comments=""):
        self.name_ = name
        self.begin_comments = begin_comments.strip(" \t")
        self.end_comments = end_comments.strip(" \t")
        self.contents = OrderedDict()

    @staticmethod
    def from_lines(lines):
        line, begin_comments = parse_next_line(lines)
        name = line[6:].lower()
        if name == "output":
            section = HTCOutputSection(name, begin_comments)
        elif name.startswith("output_at_time"):
            section = HTCOutputAtTimeSection(name, begin_comments)
        else:
            section = HTCSection(name, begin_comments)
        while lines:
            if lines[0].lower().startswith("begin"):
                section._add_contents(HTCSection.from_lines(lines))
            elif lines[0].lower().startswith("end"):
                line, section.end_comments = parse_next_line(lines)
                break
            else:
                section._add_contents(section.line_from_line(lines))
        else:
            raise Exception("Section '%s' has not end" % section.name_)
        return section

    def line_from_line(self, lines):
        return HTCLine.from_lines(lines)

    def __str__(self, level=0):
        s = "%sbegin %s;%s\n" % ("  " * level, self.name_, (("", "\t" + self.begin_comments)
                                                            [bool(self.begin_comments.strip())]).replace("\t\n", "\n"))
        s += "".join([c.__str__(level + 1) for c in self])
        s += "%send %s;%s\n" % ("  " * level, self.name_, (("", "\t" + self.end_comments)
                                                           [self.end_comments.strip() != ""]).replace("\t\n", "\n"))
        return s

    def get_subsection_by_name(self, name, field='name'):
        lst = [s for s in self if field in s and s[field][0] == name]
        if len(lst) == 1:
            return lst[0]
        else:
            if len(lst) == 0:
                raise ValueError("subsection '%s' not found" % name)
            else:
                raise NotImplementedError()


class HTCLine(HTCContents):
    values = None
    comments = ""

    def __init__(self, name, values, comments):
        if "__" in name:
            name = name[:name.index("__")]
        self.name_ = name
        self.values = list(values)
        self.comments = comments.strip(" \t")

    def __repr__(self):
        return str(self)

    def __str__(self, level=0):
        if self.name_ == "":
            return ""
        return "%s%s%s;%s\n" % ("  " * (level), self.name_,
                                ("", "\t" + self.str_values())[bool(self.values)],
                                ("", "\t" + self.comments)[bool(self.comments.strip())])

    def str_values(self):
        return " ".join([str(v).lower() for v in self.values])

    def __getitem__(self, key):
        try:
            return self.values[key]
        except:
            raise IndexError("Parameter %s does not exists for %s" % (key + 1, self.location()))

    @staticmethod
    def from_lines(lines):
        line, end_comments = parse_next_line(lines)
        if len(line.split()) > 0:
            _3to2list3 = list(line.split())
            name, values, = _3to2list3[:1] + [_3to2list3[1:]]
        else:
            name = line
            values = []

        values = [fmt_value(v) for v in values]
        return HTCLine(name, values, end_comments)

    def remove(self):
        self.name_ = ""
        self.values = []
        self.comments = ""

    def compare(self, other):
        s = ""
        if self.values != other.values:
            s += "\n".join(["+ %s" % l for l in str(self).strip().split("\n")]) + "\n"
            s += "\n".join(["- %s" % l for l in str(other).strip().split("\n")]) + "\n"
            s += "\n"
        return s


class HTCOutputSection(HTCSection):
    sensors = None

    def __init__(self, name, begin_comments="", end_comments=""):
        HTCSection.__init__(self, name, begin_comments=begin_comments, end_comments=end_comments)
        self.sensors = []

    def add_sensor(self, type, sensor, values=[], comment="", nr=None):
        self._add_sensor(HTCSensor(type, sensor, values, comment), nr)

    def _add_sensor(self, htcSensor, nr=None):
        if nr is None:
            nr = len(self.sensors)
        self.sensors.insert(nr, htcSensor)

    def line_from_line(self, lines):
        while len(lines) and lines[0].strip() == "":
            lines.pop(0)
        name = lines[0].split()[0].strip()
        if name in ['filename', 'data_format', 'buffer', 'time']:
            return HTCLine.from_lines(lines)
        else:
            return HTCSensor.from_lines(lines)

    def _add_contents(self, contents):
        if isinstance(contents, HTCSensor):
            self._add_sensor(contents)
        else:
            return HTCSection._add_contents(self, contents)

    def __str__(self, level=0):
        s = "%sbegin %s;%s\n" % ("  " * level, self.name_, ("", "\t" + self.begin_comments)
                                 [len(self.begin_comments.strip()) > 0])
        s += "".join([c.__str__(level + 1) for c in self])
        s += "".join([s.__str__(level + 1) for s in self.sensors])
        s += "%send %s;%s\n" % ("  " * level, self.name_, ("", "\t" + self.end_comments)
                                [self.end_comments.strip() != ""])
        return s

    def compare(self, other):
        s = HTCContents.compare(self, other)
        for s1, s2 in zip(self.sensors, other.sensors):
            s += s1.compare(s2)
        for s1 in self.sensors[len(other.sensors):]:
            s += "\n".join(["- %s" % l for l in str(s1).strip().split("\n")]) + "\n"
        for s2 in self.sensors[len(self.sensors):]:
            s += "\n".join(["- %s" % l for l in str(s2).strip().split("\n")]) + "\n"

        return s


class HTCOutputAtTimeSection(HTCOutputSection):
    type = None
    time = None

    def __init__(self, name, begin_comments="", end_comments=""):
        if len(name.split()) < 3:
            raise ValueError('"keyword" and "time" arguments required for output_at_time command:\n%s' % name)
        name, self.type, time = name.split()
        self.time = float(time)
        HTCOutputSection.__init__(self, name, begin_comments=begin_comments, end_comments=end_comments)

    def __str__(self, level=0):
        s = "%sbegin %s %s %s;%s\n" % ("  " * level, self.name_, self.type, self.time,
                                       ("", "\t" + self.begin_comments)[len(self.begin_comments.strip())])
        s += "".join([c.__str__(level + 1) for c in self])
        s += "".join([s.__str__(level + 1) for s in self.sensors])
        s += "%send %s;%s\n" % ("  " * level, self.name_, ("", "\t" + self.end_comments)
                                [self.end_comments.strip() != ""])
        return s


class HTCSensor(HTCLine):
    type = ""
    sensor = ""
    values = []

    def __init__(self, type, sensor, values, comments):
        self.type = type
        self.sensor = sensor
        self.values = values
        self.comments = comments.strip(" \t")

    @staticmethod
    def from_lines(lines):
        line, comments = parse_next_line(lines)
        if len(line.split()) > 2:
            _3to2list5 = list(line.split())
            type, sensor, values, = _3to2list5[:2] + [_3to2list5[2:]]
        elif len(line.split()) == 2:
            type, sensor = line.split()
            values = []
        else:
            type, sensor, values = "", "", []

        def fmt(v):
            try:
                if int(float(v)) == float(v):
                    return int(float(v))
                return float(v)
            except ValueError:
                return v
        values = [fmt(v) for v in values]
        return HTCSensor(type, sensor, values, comments)

    def __str__(self, level=0):
        return "%s%s %s%s;%s\n" % ("  " * (level),
                                   self.type,
                                   self.sensor,
                                   ("", "\t" + self.str_values())[bool(self.values)],
                                   ("", "\t" + self.comments)[bool(self.comments.strip())])

    def compare(self, other):
        s = ""
        if self.sensor != other.sensor or self.values != other.values:
            s += "\n".join(["+ %s" % l for l in str(self).strip().split("\n")]) + "\n"
            s += "\n".join(["- %s" % l for l in str(other).strip().split("\n")]) + "\n"
            s += "\n"
        return s



'''
Created on 20/01/2014

@author: MMPE

See documentation of HTCFile below

'''
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from builtins import zip
from builtins import int
from builtins import str
from future import standard_library
import os


standard_library.install_aliases()



class HTCDefaults(object):


    empty_htc = """begin simulation;
        time_stop 600;
        solvertype    1;    (newmark)
        on_no_convergence continue;
        convergence_limits 1E3 1.0 1E-7; ; . to run again, changed 07/11
        begin newmark;
          deltat    0.02;
        end newmark;
    end simulation;
    ;
    ;----------------------------------------------------------------------------------------------------------------------------------------------------------------
    ;
    begin new_htc_structure;
      begin orientation;
      end orientation;
      begin constraint;
      end constraint;
    end new_htc_structure;
    ;
    ;----------------------------------------------------------------------------------------------------------------------------------------------------------------
    ;
    begin wind ;
      density                 1.225 ;
      wsp                     10   ;
      tint                    1;
      horizontal_input        1     ;            0=false, 1=true
      windfield_rotations     0 0.0 0.0 ;    yaw, tilt, rotation
      center_pos0             0 0 -30 ; hub heigth
      shear_format            1   0;0=none,1=constant,2=log,3=power,4=linear
      turb_format             0     ;  0=none, 1=mann,2=flex
      tower_shadow_method     0     ;  0=none, 1=potential flow, 2=jet
    end wind;
    ;
    ;----------------------------------------------------------------------------------------------------------------------------------------------------------------
    ;
    begin dll;
    end dll;
    ;
    ;----------------------------------------------------------------------------------------------------------------------------------------------------------------
    ;
    begin output;
      general time;
    end output;
    exit;"""


    def add_mann_turbulence(self, L=29.4, ae23=1, Gamma=3.9, seed=1001, high_frq_compensation=True,
                            filenames=None,
                            no_grid_points=(16384, 32, 32), box_dimension=(6000, 100, 100),
                            dont_scale=False,
                            std_scaling=None):
        wind = self.add_section('wind')
        wind.turb_format = 1
        mann = wind.add_section('mann')
        if 'create_turb_parameters' in mann:
            mann.create_turb_parameters.values = [L, ae23, Gamma, seed, int(high_frq_compensation)]
        else:
            mann.add_line('create_turb_parameters', [L, ae23, Gamma, seed, int(high_frq_compensation)], "L, alfaeps, gamma, seed, highfrq compensation")
        if filenames is None:
            
            import numpy as np
            dxyz = tuple(np.array(box_dimension) / no_grid_points)
            from wetb.wind.turbulence import mann_turbulence
            filenames = ["./turb/" + mann_turbulence.name_format % ((L, ae23, Gamma, high_frq_compensation) + no_grid_points + dxyz + (seed, uvw)) for uvw in ['u', 'v', 'w']]
        if isinstance(filenames, str):
            filenames = ["./turb/%s_s%04d%s.bin" % (filenames, seed, c) for c in ['u', 'v', 'w']]
        for filename, c in zip(filenames, ['u', 'v', 'w']):
            setattr(mann, 'filename_%s' % c, filename)
        for c, n, dim in zip(['u', 'v', 'w'], no_grid_points, box_dimension):
            setattr(mann, 'box_dim_%s' % c, "%d %.4f" % (n, dim / (n - 1)))
        if dont_scale:
            mann.dont_scale = 1
        else:
            try:
                del mann.dont_scale
            except KeyError:
                pass
        if std_scaling is not None:
            mann.std_scaling = "%f %f %f" % std_scaling
        else:
            try:
                del mann.std_scaling
            except KeyError:
                pass
            


    def add_turb_export(self, filename="export_%s.turb", samplefrq = None):
        exp = self.wind.add_section('turb_export', allow_duplicate=True)
        for uvw in 'uvw':
            exp.add_line('filename_%s'%uvw, [filename%uvw])
        sf = samplefrq or max(1,int( self.wind.mann.box_dim_u[1]/(self.wind.wsp[0] * self.deltat())))
        exp.samplefrq = sf
        if "time" in self.output:
            exp.time_start = self.output.time[0]
        else:
            exp.time_start = 0
        exp.nsteps = (self.simulation.time_stop[0]-exp.time_start[0]) / self.deltat()
        for vw in 'vw':
            exp.add_line('box_dim_%s'%vw, self.wind.mann['box_dim_%s'%vw].values)

        

    def import_dtu_we_controller_input(self, filename):
        dtu_we_controller = [dll for dll in self.dll if dll.name[0] == 'dtu_we_controller'][0]
        with open (filename) as fid:
            lines = fid.readlines()
        K_r1 = float(lines[1].replace("K = ", '').replace("[Nm/(rad/s)^2]", ''))
        Kp_r2 = float(lines[4].replace("Kp = ", '').replace("[Nm/(rad/s)]", ''))
        Ki_r2 = float(lines[5].replace("Ki = ", '').replace("[Nm/rad]", ''))
        Kp_r3 = float(lines[7].replace("Kp = ", '').replace("[rad/(rad/s)]", ''))
        Ki_r3 = float(lines[8].replace("Ki = ", '').replace("[rad/rad]", ''))
        KK = lines[9].split("]")
        KK1 = float(KK[0].replace("K1 = ", '').replace("[deg", ''))
        KK2 = float(KK[1].replace(", K2 = ", '').replace("[deg^2", ''))
        cs = dtu_we_controller.init
        cs.constant__11.values[1] = "%.6E" % K_r1
        cs.constant__12.values[1] = "%.6E" % Kp_r2
        cs.constant__13.values[1] = "%.6E" % Ki_r2
        cs.constant__16.values[1] = "%.6E" % Kp_r3
        cs.constant__17.values[1] = "%.6E" % Ki_r3
        cs.constant__21.values[1] = "%.6E" % KK1
        cs.constant__22.values[1] = "%.6E" % KK2


class HTCExtensions(object):
    def get_shear(self):
        shear_type, parameter = self.wind.shear_format.values
        z0 = -self.wind.center_pos0[2]
        wsp = self.wind.wsp[0]
        if shear_type==1: #constant
            return lambda z : wsp
        elif shear_type==3:
            from wetb.wind.shear import power_shear
            return power_shear(parameter, z0, wsp)
        else:
            raise NotImplementedError